--------------------------------------------------------------------------------------------------------------
-- TITLE           :   SQL components used to test DAO execution of the "Template WS" project.
-- LAST MODIFIED   :   25/04/2012
-- AUTHOR          :   S. ADRIANI
-- VERSION         :   1.0
-- REFERENCE       :   Adriani's Oracle Project (AOP): a tutorial for building a JSP-Oracle benchmark
--                 :   (Personal remark: consider this SQL code as branch of the AOP's trunk)
-- TO DO           :   Nothing
--------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION GET_ENTITY_TIME(entity_label IN VARCHAR2, entidy_id IN NUMBER)
RETURN NUMBER IS
--------------------------------------------------------------------------------------------------------------
-- FUNCTION            :   GET_ENTITY_TIME
-- LAST MODIFIED       :   25/04/2012
-- AUTHOR              :   S. ADRIANI
-- DESCRIPTION         :   Return the elapsed time for the input entity, matching label OR id.
--                     :   If both input parameters are given, the record must match both parameter.
--                     :   However it's possibile to pass one Of them as a NULL value

-- INPUT PARAMETERS
-- entity_label        :   The label of the entity, e.g. "Benchmark_query_1"
-- entidy_id           :   The ID of the entity (an integer number)

-- OUTPUT PARAMETERS
-- modem_status        :   The elapsed time for the entity, i.e. END_TIME - START_TIME

-- REMARK              :   The result of this function could be obtained by a simple "old plain SQL query":
--                     :   here we used a function just to show some basic PL/SQL features
--------------------------------------------------------------------------------------------------------------

  -- Inner variable
  range_1           NUMBER ;

  -- Debug variables
  verbose           BOOLEAN := TRUE ;
  error_code		VARCHAR2(12) ;
  error_message		VARCHAR2(300) ;

BEGIN

  -- 1. If we have both parameters we try a double match
  IF (entity_label IS NOT NULL) AND (entidy_id IS NOT NULL) THEN

   SELECT (
    (extract(hour from END_TIME) - extract(hour from START_TIME))*3600+
    (extract(minute from END_TIME) - extract(minute from START_TIME))*60+
     extract(second from END_TIME) - extract(second from START_TIME)
	) SEC
    INTO range_1
    FROM ENTITY_TIMES
    WHERE ID_ENTITY = entidy_id
    AND LABEL = entity_label ;

    -- 1.1. Optional: showing result on the server STDOUT
    IF (verbose) THEN
      DBMS_OUTPUT.PUT_LINE('Using double match') ;
    END IF ;

  -- 2.1 Else we try a single match
  ELSE

	SELECT (
    (extract(hour from END_TIME) - extract(hour from START_TIME))*3600+
    (extract(minute from END_TIME) - extract(minute from START_TIME))*60+
     extract(second from END_TIME) - extract(second from START_TIME)
	) SEC
    INTO range_1
    FROM ENTITY_TIMES
    WHERE ID_ENTITY = entidy_id
    OR LABEL = entity_label ;

    -- 2.1. Optional: showing result on the server STDOUT
    IF (verbose) THEN
      DBMS_OUTPUT.PUT_LINE('Using single match') ;
    END IF ;

  END IF ;

  -- 3. Optional: showing result on the server STDOUT
  IF (verbose) THEN
    DBMS_OUTPUT.PUT_LINE('Found range : '|| range_1) ;
  END IF ;

  -- 4. Returning result
  RETURN range_1 ;

-- 5. Handling a global exception
EXCEPTION

  WHEN OTHERS THEN
    error_code := SQLCODE ;
    error_message := SUBSTR(SQLERRM, 1, 255)||'; General error for input:'||entity_label||'-'||entidy_id ;
    DBMS_OUTPUT.PUT_LINE('[PUBLIC PACKAGE] - GET_ENTITY_TIME'||ora_sysevent||error_code||error_message) ;
    RETURN NULL ;

END GET_ENTITY_TIME ;

/*
--------------------------------------------------------------------------------------------------------------
-- OPTIONAL: SQL STATEMENT FOR TESTING/DEBUG ON A SQL CLIENT
--------------------------------------------------------------------------------------------------------------

SET SERVEROUTPUT ON;
EXEC DBMS_OUTPUT.ENABLE(1000000);
SELECT GET_ENTITY_TIME('A general entity', 4) OUTPUT FROM DUAL ;
SELECT GET_ENTITY_TIME('A general entity', NULL) OUTPUT FROM DUAL ;
SELECT GET_ENTITY_TIME(null, 4) OUTPUT FROM DUAL ;

*/