--------------------------------------------------------------------------------------------------------------
-- TITLE           :   SQL components used to test DAO execution of the "Template WS" project.
-- LAST MODIFIED   :   25/04/2012
-- AUTHOR          :   S. ADRIANI
-- VERSION         :   1.0
-- REFERENCE       :   Adriani's Oracle Project (AOP): a tutorial for building a JSP-Oracle benchmark
--                 :   (Personal remark: consider this SQL code as branch of the AOP's trunk)
-- TO DO           :   Nothing
--------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION GET_ALL_TIMES(entity_label IN VARCHAR2, entidy_id IN NUMBER)
RETURN T_ALL_TIMES_LIST PIPELINED IS
--------------------------------------------------------------------------------------------------------------
-- FUNCTION            :   GET_ALL_TIMES
-- LAST MODIFIED       :   25/04/2012
-- AUTHOR              :   S. ADRIANI
-- DESCRIPTION         :   Return the first 4 times for the entity matching label and having ID greater then the input ID

-- INPUT PARAMETERS
-- entity_label        :   The label of the entity, e.g. "Benchmark_query_1"
-- entidy_id           :   The minimum ID of the entity (an integer number)

-- OUTPUT PARAMETERS   :   A recordset with 2 columns. The first colums is the start time, the second the end time.
--                     :   The recordset has a maximum size of 4 records (for training reasons)

-- REMARK              :   The result of this function could be obtained by a simple "old plain SQL query":
--                     :   here we used a function just to show some basic PL/SQL features
--------------------------------------------------------------------------------------------------------------

  -- Local variables
  n                 INTEGER ;

  -- Varrays definitions
  TYPE start_varray  IS VARRAY(4) OF DATE ;
  TYPE end_varray    IS VARRAY(4) OF DATE ;
  TYPE label_varray  IS VARRAY(4) OF VARCHAR2(8) NOT NULL ;

  -- Varray declaration
  start_date    START_VARRAY    := start_varray (NULL, NULL, NULL, NULL) ;
  end_date      END_VARRAY      := end_varray   (NULL, NULL, NULL, NULL) ;
  desc_label    LABEL_VARRAY    := label_varray ('First', 'Second', 'Third', 'Fourth') ;

  -- Debug variables
  verbose           BOOLEAN := TRUE ;
  error_code		VARCHAR2(12) ;
  error_message		VARCHAR2(300) ;

  -- Cursors
  CURSOR my_cursor IS

    SELECT START_TIME, END_TIME
    FROM ENTITY_TIMES
    WHERE LABEL = entity_label
	AND ID_ENTITY >= entidy_id
	AND ROWNUM < 5 ;

BEGIN

    -- 1. Retrieving data and producing pipeline
    BEGIN

	    -- 1.1 Fetching records using a cursor
		n := 0 ;
        OPEN my_cursor ;
        LOOP
            n := n + 1 ;
            FETCH my_cursor INTO start_date(n), end_date(n) ;
            EXIT WHEN my_cursor%NOTFOUND ;
        END LOOP;
        CLOSE my_cursor ;
		n := n - 1 ;

        -- 1.2. Optional: showing result on the server STDOUT
        IF (verbose) THEN
          DBMS_OUTPUT.PUT_LINE('N value after loop: '||n) ;
        END IF ;

		-- 1.3. Filling output pipeline
		FOR i IN 1..n LOOP
		  PIPE ROW ( ALL_TIMES_LIST( desc_label(i), start_date(i), end_date(i)   ) )  ;
		END LOOP ;

		-- 1.4. Handling exception for this begin-end code-block
        EXCEPTION
          WHEN OTHERS THEN
          error_code := SQLCODE ;
          error_message := SUBSTR(SQLERRM, 1, 255)||'; Error fetching data for input:'||entity_label||'-'||entidy_id ;
          DBMS_OUTPUT.PUT_LINE('[PUBLIC PACKAGE] - GET_ALL_TIMES'||ora_sysevent||error_code||error_message) ;

	END ;

-- 2. Handling the eventual global exception
  EXCEPTION

    WHEN OTHERS THEN
    error_code := SQLCODE ;
    error_message := SUBSTR(SQLERRM, 1, 255)||'; General error for input:'||entity_label||'-'||entidy_id ;
    DBMS_OUTPUT.PUT_LINE('[PUBLIC PACKAGE] - GET_ALL_TIMES'||ora_sysevent||error_code||error_message) ;

END GET_ALL_TIMES ;

/*
--------------------------------------------------------------------------------------------------------------
-- OPTIONAL: SQL STATEMENT FOR TESTING/DEBUG ON A SQL CLIENT
--------------------------------------------------------------------------------------------------------------

SET SERVEROUTPUT ON;
EXEC DBMS_OUTPUT.ENABLE(1000000);
SELECT * FROM TABLE(GET_ALL_TIMES('A general entity', 4));
SELECT * FROM TABLE(GET_ALL_TIMES('Benchmark_query_1', 5));

*/