/**
* Title: 		UnitTester
* Description:	Class showing some examples of unit testing launched by the SimpleClient
* Copyright:    Copyright (c) 2008-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version		1.0
*/

package adriani.jws.template.client ;

import adriani.jws.template.Settings ;
import adriani.jws.template.db.Database ;
import adriani.jws.template.db.ServiceDAO ;
import adriani.jws.template.service.ServiceMacro ;
import adriani.jws.template.service.ServiceLogic ;
import adriani.jws.template.util.* ;

import java.util.* ;
import java.math.* ;

/**
* The methods offered by this class can be used by the <code>SimpleClient</code> to execute some short
* unit test on specific code-blocks during development. This is the right place where to add, delete or modify
* methods without worrying about the web service architecture. To invoke the required code block just uncomment
* the last line of the <code>main()</code> method in the <code>SimpleClient</code> class.<br/><br/>
*
* <strong>Remark</strong>: to grant no coupling with the other classes, all the methods of this class must
* take care to log results on STDOUT or on file: hence the method return type MUST NOT contain any
* reference to a class of the Template web service.<br/><br/>
*
* <strong>Note</strong>: it's not necessary to include this class into any final release: keep this class
* into the final AAR package only if you think that some tested code-block could be useful in the future.
*/
public class UnitTester {

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String				leadingName 	= "UnitTester - " ;

  	/** Retrieving the logger instance to be used for this class (static) .									*/
	protected static 		WSLogger			logger 			= WSLogger.getLogger(UnitTester.class) ;

    /**
	* Choose the unit tester to be executed. Current implementation offer following tests:<br/><br/>
	*
	* 0. Empty method: place holder for testing short code blocks on the fly.<br/>
	* 1. Show Java Properties: print all Java Properties.<br/>
	* 2. Empty method: place holder for testing calls to PL/SQL procedures.<br/>
	* 3. Multi threading: test the behavior of the logger in the multithreading environment.<br/>
	* 4. Rosen combinations: test the Rosen combinations algorithm in the range </em>1..n</em>.<br/>
	* 5. Rosen zero combinations: test the Rosen combinations algorithm in the range </em>0..n</em>.<br/>
	*
	* @param the integer code selecting the test
	* @param param the input args string array converted into a <code>StringBuffer[]</code> object
    */
    public void runTest(int code, StringBuffer[] param) {

		switch(code) {
			case 	0	: simpleTest(param) ;						break ;
			case 	1	: showProperties() ;						break ;
			case 	2	: testProcedure(param) ;					break ;
			case 	3	: multiThreading() ;						break ;
			case 	4	: rosenCombinations(5, 3) ;					break ;
			case 	5	: rosenZeroCombinations(5, 3) ;				break ;
			default		: System.out.println(leadingName + "Please specify a number from 0 to 5") ;
		}
	}

// ************************************************************************************************************
// ********************************				GENERAL TEST			***************************************
// ************************************************************************************************************

    /**
	* General test, used as a place-holder to write brand new code-block. After testing, usually this code-block
	* is moved and renamed, creating a new method for the <code>UnitTester</code> class, or is discharged.
	*
	* @param buffer the input args string array converted into a <code>StringBuffer[]</code> object
    */
    public void simpleTest(StringBuffer[] buffer) {

		System.out.println(leadingName + "Running simple test" 	) ;
		System.out.println(leadingName + "(nothing to do)" 		) ;
	}

// ************************************************************************************************************
// ********************************				SYSTEM TEST				***************************************
// ************************************************************************************************************

    /**
	* Print on the STDOUT all the existing system properties used by the Java Runtime.
    */
    public void showProperties() {

		int 		n = 1 ;
		Enumeration e = System.getProperties().propertyNames() ;
		System.out.println() ;

		while ( e.hasMoreElements() ) {
			String name = (String) e.nextElement() ;
			System.out.println(n++ + ") " + name + ": " + System.getProperty(name)) ;
		}
	}

    /**
	* Test the multi-threading behavior of the logger system.
    */
    public void multiThreading() {

		// Printing test header
		int max = 3 ;
		System.out.println(leadingName + "Running multithreading test (" + max + ")\n" ) ;

		//Creating the array containing all threads
		HelperThread[] singleThread = new HelperThread[max] ;
		for (int i = 0 ; i < max ; i++) singleThread[i] = new HelperThread(0) ;

		// Starting all threads
		for (int i = 0 ; i < max ; i++) singleThread[i].start() ;

		// Running all threads for the specified number of seconds
		try 							{	Thread.sleep(500) ; 								}
		catch(InterruptedException e) 	{	System.out.println("Sleep interrupted ...") ; 		}

		// Stopping all threads
		for (int i = 0 ; i < max ; i++) singleThread[i].running = false ;
	}

// ************************************************************************************************************
// ********************************			TESTING DATABASE			***************************************
// ************************************************************************************************************

    /**
	* Test a PL/SQL stored procedure.
	*
	* @param buffer the input args string array converted into a <code>StringBuffer[]</code> object
    */
    public void testProcedure(StringBuffer[] param) {

		System.out.println("\nNOT IMPLEMENTED") ;
	}

// ************************************************************************************************************
// ********************************			TESTING ALGORITHIMCS				*******************************
// ************************************************************************************************************

    /**
	* Print all the combinations of r elements from a set of n elements.
	* The result is printed as an <code>int</code> array using a index starting from <strong>1</strong>.
	* This means that the first element of the array MUST be ignored.
	*
	* @param n the total number of elements in the set
	* @param r the number of elements to be chosen for each single combination
    */
    public void rosenCombinations(int n, int r) {

		if (r > n) throw new IllegalArgumentException() ;
		if (n < 1) throw new IllegalArgumentException() ;

		System.out.println() ;
		System.out.println(leadingName + "Testing Rosen combinations with values: " + r + " " + n) ;
		System.out.println("(ignore first element of any array)\n") ;

		BigInteger total;
		BigInteger numLeft;
		int[] a = new int[r+1] ;

		BigInteger nFact 		= this.getFactorial(n) ;
		BigInteger rFact 		= this.getFactorial(r) ;
		BigInteger nminusrFact 	= this.getFactorial (n - r) ;
		total = nFact.divide (rFact.multiply(nminusrFact)) ; 	// Total combination number = n! / (r! * (n-k)!)

		// Resetting result array
	    for (int i = 1; i < a.length; i++) a[i] = i ;
		numLeft = new BigInteger( total.toString () ) ;

		// Main cycle, performing while (numLeft > 0)
		while (numLeft.compareTo (BigInteger.ZERO) == 1) {

			if ( numLeft.equals(total) ) {
			  numLeft = numLeft.subtract(BigInteger.ONE) ;
			  printArray(a) ;
			} else {
				int i = r ;
				while (a[i] == n - r + i) i-- ;
				a[i] = a[i] + 1 ;
				for (int j = i ; j <= r; j++) a[j] = a[i] + j - i ;
				numLeft = numLeft.subtract(BigInteger.ONE) ;
				printArray(a) ;
			}

		}
	}

    /**
	* Print all the combinations of r elements from a set of n elements.
	* The result is printed as an <code>int</code> array using a index starting from <strong>0</strong>.
	*
	* @param n the total number of elements in the set
	* @param r the number of elements to be chosen for each single combination
    */
    public void rosenZeroCombinations(int n, int r) {

		if (r > n) throw new IllegalArgumentException() ;
		if (n < 1) throw new IllegalArgumentException() ;

		System.out.println() ;
		System.out.println(leadingName + "Testing Rosen Zero combinations with values: " + r + " " + n) ;
		System.out.println() ;

		BigInteger total;
		BigInteger numLeft;
		int[] a = new int[r] ;

		BigInteger nFact 		= this.getFactorial(n) ;
		BigInteger rFact 		= this.getFactorial(r) ;
		BigInteger nminusrFact 	= this.getFactorial (n - r) ;
		total = nFact.divide (rFact.multiply(nminusrFact)) ; 	// Total combination number = n! / (r! * (n-k)!)

		// Resetting result array
	    for (int i = 0; i < a.length; i++) a[i] = i ;
		numLeft = new BigInteger( total.toString() ) ;

		// Main cycle, performing while (numLeft > 0)
		while (numLeft.compareTo (BigInteger.ZERO) == 1) {

			if ( numLeft.equals(total) ) {
			  numLeft = numLeft.subtract(BigInteger.ONE) ;
			  printArray(a) ;
			} else {
				int i = r - 1 ;
				// Starting always from the last array element and proceeding backwards
				// until we find a not sequential number (e.g. if a = {0,1,2,5,6,7} stops at "2")
				while (a[i] == n - r + i) i-- ;
				// Incrementing only the CURRENT ELEMENT, focused by previous while loop
				// In the previous example we'll get a = {0,1,3,5,6,7}
				a[i] = a[i] + 1 ;
				// Now OVERWRITING all elements after the current element, creating a sequence.
				// In the previous example we'll get a = {0,1,3,4,5,6}
				for (int j = i + 1; j < r; j++) a[j] = a[i] + j - i ;
				numLeft = numLeft.subtract(BigInteger.ONE) ;
				printArray(a) ;
			}

		}
	}

    /**
	* Return the factorial number produced by the input integer.
	*
	* @param n an integer number
	* @return the factorial computation, i.e. <em>n!</em>
    */
    public static BigInteger getFactorial(int n) {

      BigInteger fact = BigInteger.ONE ;
      for (int i = n; i > 1; i--) fact = fact.multiply (new BigInteger (Integer.toString (i))) ;
      return fact ;
	}

// ************************************************************************************************************
// ********************************					UTILITIES				***********************************
// ************************************************************************************************************

    /**
	* Print on the STDOUT all the integer values contained in the input array.
	*
	* @param array the array to be printed
    */
    public void printArray(int[] array) {

		System.out.print("Array: .") ;
		for (int i = 0 ; i < array.length ; i++) System.out.print(array[i] + ".") ;
		System.out.println() ;
	}

} // End of class

// **********************************************************************************************************
// ************************					HELPER CLASS			*****************************************
// **********************************************************************************************************

/**
* Simple thread used to test the multi-threading behavior of the web services on Axis2.
*/
class HelperThread extends Thread {

	/** Sleeping time in seconds 																				*/
	private 				long 		sleepTime ;

	/** The boolean flag used to check condition of the endless loop in the <code>run()</code> method.			*/
	protected 				boolean 	running ;

  	/** Beginning of each logging message produced by this class.											*/
	protected 				String 		leadingName ;

	/**
	* Constructor.
	*
	* @param sleepTime the sleep time for this thread, in milliseconds
	*/
	HelperThread(int sleepTime) {

		this.running = true ;
		this.sleepTime = sleepTime ;
		String name = "HelperThread:" + this.getId() ;
		this.setName(name) ;
		this.leadingName = name + " - ";
	}

	/**
	* Endless loop, starting when we call the <code>helperThread.start()</code> method.
	* <br/>
	* To stop the endless loop it's necessary to set <code>helperThread.running = false</code>.
	*/
	public void run() {

		System.out.println(leadingName + "Started with timeout: " + sleepTime + " ms") ;

		while (this.running) {

			// Sleeping for all the sleepTime cycle
			try 							{	sleep(sleepTime) ; 											}
			catch (InterruptedException e)	{ 	System.err.println(leadingName + "Interrupted...") ;	 	}
			// Logging
			UnitTester.logger.logInfo(leadingName + "Running") ;
		}

		System.out.println(leadingName + "Exiting") ;
	}

} // End of helper class