/**
* Title: 		ServiceDAO
* Description:	The DataBase Accessor Object (implementing the Data Access Object layer)
* Copyright:    Copyright (c) 2010-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.db ;

import adriani.jws.template.* ;
import adriani.jws.template.service.* ;
import adriani.jws.template.util.* ;
import adriani.jws.template.wrappers.*;

import java.sql.* ;
import java.util.* ;

/**
* The class wrapping all the SQL queries and statement necessary to retrieve data from the database(s).
* This class stores the necessary references to the database connections. These fields are not present in the
* <code>GeneralDAO</code> parent class because usually it's better to avoid storing DB connection as class fields.
* Here we store the connections to improve performances by sharing the same connection only when invoking
* a sequence of queries from the <code>ServiceMacro</code>, usually during the same SOAP call.<br/><br/>
*
* The connections are obtained from the <code>Database</code> class using the <code>MockConnection</code>
* wrapper, which allows to perform some specific test/debug operations and could improve performances when
* using prepared statements in a connection pooling environment.<br/>
* Note that it's up to the <code>Database</code> class to manage the connections pooling (if used), in order
* to keep uncoupled the implementation of this class from the management of connections:
* no matter which pool class is used, a DAO class should always retrieve connections just calling:<br/><br/>
*
* <code>Database.getMockConnection(targetDB)</code><br/><br/>
*
* and returning them by calling:<br/><br/>
*
* <code>Database.returnMockConnection(targetDB, this.mock[targetDB])</code>
*/
public class ServiceDAO extends GeneralDAO {

  	/** The array of mock connections for the necessary database(s) (accessed by this DAO).					*/
	protected 				MockConnection[]	mock ;

  	/** A reference to the <code>ServiceMacro</code> object which created this DA0.							*/
	protected 				ServiceMacro 		serviceMacro  ;
	
  	/** SAMPLE query in string format used to create the corresponding prepared statement.					*/
	protected static final 	String 				GET_CUSTOMER 			= "SELECT NAME, SURNAME FROM CUSTOMERS WHERE TELEPHONE = ? AND ROWNUM < 2" ;

	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	protected static final	StringBuffer		ONE						= new StringBuffer("1") ;

	/** Retrieving the logger instance to be used for this class.											*/
	protected static 		WSLogger			logger 					= WSLogger.getLogger(ServiceDAO.class) ;

	/**
	* This constructor DOES NOT require (nor create) the <code>serviceMacro</code> object, hence the
	* configuration files and properties will NOT be loaded. Default values (hard coded) will be used
	* instead, as defined in the <code>Settings</code> class.
    */
    public ServiceDAO() {

		this.leadingName 	= "Service DAO - " ;
		this.mock 			= new MockConnection[Database.DB_NUMBER] ;
	}

	/**
	* Stateful constructor.
	*
	* @param macro the <code>ServiceMacro</code> class creating and using this DAO
    */
    public ServiceDAO(ServiceMacro macro) {

		this() ;
		this.serviceMacro = macro ;
	}

// **********************************************************************************************************
// *****************************		CONNECTION MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Open the connection to the target database. The connection to be opened is selected by the input
	* <code>targetDB</code> parameter, for example:<br/><br/>
	*
	* targetDB = DB_01 --> open a connection to the DB_01 database.<br/>
	* targetDB = DB_02 --> open a connection to the DB_02 database.<br/>
	* etc...<br/><br/>
	*
	* Connections are retrieved from the <code>Database</code> class using the <code>getMockConnection</code>
	* mechanism, which allowS to retrieve both simple connections or wrapped connections.<br/>
	* Current implementation immediately QUIT execution if we get an error opening the connection,
	* logging a SEVERE error message and returning <code>false</code>. Usually this means that
	* the web service will not produce a valid response: the client could receive a SOAP response
	* containing a message like "database not found".<br/>
	*
	* @param targetDB the reference code of the database where to open the connection
	* @return <code>true</code> if the connection has been opened succesfully
	* @see Database#getMockConnection(int targetDB)
    */
    public boolean openConnection(int targetDB) {

		// 1. Checking if general Database environment is properly set
		if (mock == null) {
			logger.logSevere(leadingName + "MockConnection array not initialized") ;
			return false ;
		} else if (targetDB >= mock.length) {
			logger.logSevere(leadingName + "Database reference number " + targetDB + " can not be handled") ;
			return false ;
		}

		// 2. Trying to retrieve a valid connection wrapper
		try {
			this.mock[targetDB] = Database.getMockConnection(targetDB) ;
			return (mock[targetDB] != null) ;
		}
		catch(SQLException e) {
			logger.logSevere(leadingName + "Connection to database " + targetDB + " not found") ;
	  		e.printStackTrace() ;
			return false ;
		}
		catch(Exception e) {
			logger.logSevere(leadingName + "Java error opening connections to database " + targetDB) ;
	  		e.printStackTrace() ;
			return false ;
		}
	}

	/**
	* Close the connection specified by the input target parameter, for example:<br/><br/>
	*
	* targetDB = DB_01 --> close the connection to the DB_01 database.<br/>
	* targetDB = DB_02 --> close the connection to the DB_02 database.<br/>
	* etc...<br/><br/>
	*
	* Current implementation DOES NOT quit execution if we get an error returning the connection.
	* This means that the web service could provide a valid output, but connections to the database
	* are left "hanged up" and could cause a database crash. For this reasn such a situation is logged
	* as a SEVERE error by the log system.<br/>
	*
	* @param targetDB the number of the database where to return the connection
    */
    public void closeConnection(int targetDB) {

		if ( (mock != null) && (targetDB < mock.length) ) {
			Database.returnMockConnection(targetDB, this.mock[targetDB]) ;
			this.mock[targetDB] = null ;
		} else {
			StringBuffer message = new StringBuffer(leadingName + "Wrong database enviroment: ") ;
			message.append("Impossible to return connection for database " + targetDB) ;
			logger.logSevere(message.toString()) ;
		}
	}

	/**
	* Return the connection currently opened through the target database, for example:<br/><br/>
	*
	* targetDB = DB_01 --> close the connection to the DB_01 database.<br/>
	* targetDB = DB_02 --> close the connection to the DB_02 database.<br/>
	* etc...<br/><br/>
	*
	* @param targetDB the number of the database offering the connection
	* @return the currently opened connection to the target database
    */
    public Connection getConnection(int targetDB) {

		Connection connection = null ;

		if ( (mock != null) && (targetDB < mock.length) ) connection = mock[targetDB].getConnection() ;
		else {
			StringBuffer message = new StringBuffer(leadingName + "Wrong database enviroment: ") ;
			message.append("Impossible to use connection for database " + targetDB) ;
			logger.logSevere(message.toString()) ;
		}

		return connection ;
	}
    
// ************************************************************************************************************
// ********************************				GET DONATOR				***************************************
// *** 		This is a sample section: remove it when developing actual necessary DAO operations				***
// ************************************************************************************************************

	/**
	* Sample basic query: remove this method when developing actual DAO method. Usually we should have one
	* method like this for each query (or group of similar queries). 
	*
	* @param number the whole telephone number (without the international prefix)
	* @param customer the customer ID reference number (optional)
	* @return the customer (name, surname) pair as found in the database
    */
    public StringBuffer[][] getCustomerData(StringBuffer number, StringBuffer customer) {

		PreparedStatement 	pstmt 	= null ;
		String 				method = "mydb.getCustomerName: " ;

		try {
			System.out.println(leadingName + "Preparing:#" + number + "#") ;
			pstmt = this.getConnection(Database.DB_01).prepareStatement(GET_CUSTOMER) ;
			pstmt.setString(1, number.toString()) ;
			StringBuffer[][] result = getPreparedRecordSet(pstmt, method, 2) ;
			if ( (result != null) && (result.length > 0) ) return result ;
			else return null ;
		} catch(Exception e) {
			e.printStackTrace() ;
			logger.logSevere(leadingName + method + PSTMT_NOK + e) ;
			return null ;
		} finally {
			// This will be executed even if an exception occurs in the catch block, or after
			// the method stepped over the return statement
			try 				{ 	pstmt.close() ;															}
			catch(Exception e) 	{	logger.logSevere(leadingName + "Impossible to close pstmt " + pstmt) ;	}
		}
	}

// **********************************************************************************************************
// ************************************		PERFORMANCE ANALYSIS		*************************************
// **********************************************************************************************************

	/**
	* We override the parent (empty) method in order to register the execution status of a general SQL
	* statement in the <code>ServiceAnalyzer</code> class, by calling:<br/><br/>
	*
	* <code>serviceMacro.getServiceAnalyzer().collect(status) ;</code><br/><br/>
	*
	* @param statusValue the integer status as defined by the <code>ErrorMessage</code> class fields
	* @see ErrorMessage
	*/
	public void collectStatus(int statusValue) {

		serviceMacro.getServiceAnalyzer().collect(status) ;
	}

// ************************************************************************************************************
// *************************************			TEST & DEBUG		***************************************
// ************************************************************************************************************

	/**
	* Produces a <code>StringBuffer[][]</code> object created from some hard-coded data for a general
	* SOAP method. This method should be used only during development to obtain an output from a SOAP
	* method as long as the actual database is not available.
	*
	* @param param the array containing all the input data
	* @param howMany the number of record to be retrieved
	* @param cols the number of columns for each record
	* @return a <code>StringBuffer[][]</code> object obtained from hard-coded data
    */
    public StringBuffer[][] getDummyQuery(StringBuffer[] param, int howMany, int cols) {

		// Emualating the "no data" situation
		if ((howMany == 0) || (cols == 0)) return null ;

		// Preparing variables
		String label = "fakeQuery" ;
		if (Settings.BENCH) startQuery(label) ;
		StringBuffer[][] output = new StringBuffer[howMany][cols] ;
		System.out.println(leadingName + "getDummyQuery parameters: " + param) ;

		// Eventually printing input parameters on the STDOUT
		if (Settings.VERBOSE) {
			System.out.print(leadingName + "Running fake query using input: ") ;
			for (int i = 0 ; i < param.length ; i++) System.out.print(param[i] + " ") ;
			System.out.println() ;
		}

		// Producing the whole result set
		StringBuffer[] 	record 	= null ;
		ArrayList 		list 	= new ArrayList() ;
		for (int n = 0 ; n < howMany ; n++) {
			record = new StringBuffer[cols] ;
			// Producing a single record
			for (int i = 0 ; i < cols ; i++) {
				int index = (i < param.length) ? i : (param.length - 1) ;
				if (n == 0)	{
					record[i] = new StringBuffer("dummy"					) ;
				} else {
					record[i] = new StringBuffer("Echo " + param[index]		) ;
				}
				//System.out.println(leadingName + "Produced fake output: " + i + "->" + record[i]) ;
			}
			list.add(record) ;
		}

		// Creating output array
		int size = list.size() ;
		output = (StringBuffer[][]) list.toArray( new StringBuffer[size][cols] ) ;

		// Eventually logging results
		if (Settings.BENCH) 		endQuery(label) ;
		if (Settings.ANALYZE) 		serviceMacro.getServiceAnalyzer().collect(ErrorMessage.OK) ;
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Fake query produced: " + size + " records") ;

		return output ;
	}

} // End of class