/**
* Title: 		ErrorMessage
* Description:	Value object class bundling all messages (not only errors)
* Copyright:    Copyright (c) 2010-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;

import java.util.* ;

/**
* This class is responsible for mapping all the error codes into user-friendly text messages.
* In order to improve performance, all text messages are instantiated as STATIC class attributes, in this
* way each text message is created only once (the first time the class is loaded).
* The actual mapping between error codes and text messages is defined by some custom properties files
* having the base name <strong><code>message_list.properties</code></strong>.<br/><br/>
*
* <strong>Remark:</strong> the <code>ResourceBundle.getBundle()</code> method retrieves the base file only
* when the system Locale DOES NOT match any locale-specific file.<br/><br/>
*
* Example: consider a machine using an ENGLISH Locale and finding only two properties files:
* <code>message_list.properties</code> (base) and <code>message_list_en.properties</code> (English).
* In this case calling <code>ResourceBundle.getBundle(Settings.MESSAGES, Locale.ITALY)</code>
* will load the <code>message_list_en.properties</code> file because it matches the system Locale!
* Hence, in order to force loading of the Italian properties, it's necessary to provide also a file
* named: <code>message_list_it.properties</code>.
*/
public class ErrorMessage {

	/** General error code (business logic level)															*/
	public static final 	int 	NOK						= 1 ;
	/** General error code (business logic level)															*/
	public static final 	int 	DAO						= 2 ;
	/** General error code (business logic level)															*/
	public static final 	int 	MACRO					= 3 ;
	/** General error code (business logic level)															*/
	public static final 	int 	LOGIC					= 4 ;
	/** General error code (business logic level)															*/
	public static final 	int 	GENERAL					= 5 ;
	/** General error code (business logic level)															*/
	public static final 	int 	COMMAND					= 6 ;

	/** Database error code (DAO layer)																		*/
	public static final 	int 	SQL						= 100 ;
	/** Database error code (DAO layer)																		*/
	public static final 	int 	NO_DB					= 101 ;
	/** Database error code (DAO layer)																		*/
	public static final 	int 	NO_RECORD				= 102 ;

	/** Soap specific error code																			*/
	public static final 	int 	VALIDATION				= 200 ;

	/** Output message code, depending by specific project's requirement									*/
	public static final 	int 	OK						= 300 ;
	/** Output message code, depending by specific project's requirement									*/
	public static final 	int 	NO_RESULT				= 301 ;
	/** Output message code, depending by specific project's requirement									*/
	public static final 	int 	NO_CUSTOMER				= 302 ;
	/** Output message code, depending by specific project's requirement									*/
	public static final 	int 	BAD_DATA				= 303 ;

	/** Default value for the static field																	*/
	public static 			String 	NOK_MESSAGE 			= "RICHIESTA NON VALIDA" ;
	/** Default value for the static field																	*/
	public static 			String 	DBA_MESSAGE 			= "JAVA ERROR AT DAO LEVEL" ;
	/** Default value for the static field																	*/
	public static 			String 	MACRO_MESSAGE 			= "JAVA ERROR AT MACRO LEVEL" ;
	/** Default value for the static field																	*/
	public static	 		String 	LOGIC_MESSAGE	 		= "JAVA ERROR AT LOGIC LEVEL" ;
	/** Default value for the static field																	*/
	public static			String 	GENERAL_MESSAGE 		= "GENERAL ERRROR" ;
	/** Default value for the static field																	*/
	public static	 		String 	COMMAND_MESSAGE 		= "RECEIVED SERVICE COMMAND" ;
	/** Default value for the static field																	*/
	public static 			String 	SQL_MESSAGE 			= "SQL ERROR" ;
	/** Default value for the static field																	*/
	public static	 		String 	NODB_MESSAGE 			= "NO CONNECTION TO THE DATABASE" ;
	/** Default value for the static field																	*/
	public static 			String 	NORECORD_MESSAGE	 	= "NO RECORD" ;
	/** Default value for the static field																	*/
	public static	 		String 	VALIDATION_MESSAGE	 	= "VALIDATION FAILURE" ;
	/** Default value for the static field																	*/
	public static 			String 	DATAOK_MESSAGE 			= "RICHIESTA ACCODATA" ;
	/** Default value for the static field																	*/
	public static	 		String 	NORESULT_MESSAGE 		= "NO DATA INTO DATABASE" ;
	/** Default value for the static field																	*/
	public static 			String 	NOCUSTOMER_MESSAGE 		= "CUSTOMER NOT FOUND" ;
	/** Default value for the static field																	*/
	public static 			String 	BADDATA_MESSAGE 		= "CHIAVE ERRATA" ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 	leadingName 			= "ErrorMessage - " ;

	// Static code block
	static {

		try {
			ResourceBundle rb = ResourceBundle.getBundle(Settings.MESSAGES, Locale.ITALY) ;
			//ResourceBundle rb = ResourceBundle.getBundle(Settings.MESSAGES, Locale.ENGLISH) ;

			// Retrieving string properties
			NOK_MESSAGE				= rb.getString("error.nok"			) ;
			DBA_MESSAGE				= rb.getString("error.dao"			) ;
			MACRO_MESSAGE			= rb.getString("error.macro"		) ;
			LOGIC_MESSAGE			= rb.getString("error.logic"		) ;
			GENERAL_MESSAGE			= rb.getString("error.general"		) ;
			COMMAND_MESSAGE			= rb.getString("error.command"		) ;
			SQL_MESSAGE				= rb.getString("error.sql"			) ;
			NODB_MESSAGE			= rb.getString("error.no_db"		) ;
			NORECORD_MESSAGE		= rb.getString("error.no_record"	) ;
			VALIDATION_MESSAGE		= rb.getString("error.validation"	) ;
			DATAOK_MESSAGE			= rb.getString("output.data_ok"		) ;
			NORESULT_MESSAGE		= rb.getString("output.no_result"	) ;
			NOCUSTOMER_MESSAGE		= rb.getString("output.no_customer"	) ;
			BADDATA_MESSAGE			= rb.getString("output.bad_data"	) ;

			if (Settings.VERBOSE) System.out.println(ErrorMessage.leadingName + "Loaded multilanguage file") ;

		} catch(MissingResourceException e) {
			System.err.println(ErrorMessage.leadingName + "Message properties file not found: using defaults") ;
			e.printStackTrace() ;
		}
	}

	/**
	* Return the text message mapped into the integer code as defined in the loaded properties file.
	*
	* @param code the integer code of the error
	* @return the string text message describing the error
	*/
	public static String getMessage(int code){

		switch(code) {
			case NOK 				: return NOK_MESSAGE ;
			case DAO 				: return DBA_MESSAGE ;
			case MACRO 				: return MACRO_MESSAGE ;
			case LOGIC 				: return LOGIC_MESSAGE ;
			case GENERAL		 	: return GENERAL_MESSAGE ;
			case COMMAND		 	: return COMMAND_MESSAGE ;
			case SQL 				: return SQL_MESSAGE ;
			case NO_DB				: return NODB_MESSAGE ;
			case NO_RECORD			: return NORECORD_MESSAGE ;
			case VALIDATION			: return VALIDATION_MESSAGE ;
			case OK					: return DATAOK_MESSAGE ;
			case NO_RESULT			: return NORESULT_MESSAGE ;
			case NO_CUSTOMER		: return NOCUSTOMER_MESSAGE ;
			case BAD_DATA			: return BADDATA_MESSAGE ;
			default 				: return GENERAL_MESSAGE ;
		}
	}

} // End of class