/**
* Title: 		ServiceAnalyzer
* Description:	Custom implementation of the general <code>GeneralAnalyzer</code> class
* Copyright:    Copyright (c) 2009-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;

import java.util.* ;

/**
* Implementation of the general <code>GeneralAnalyzer</code> abstract class for the specific
* web service.<br/><br/>
*
* To provide an example of implementatio, here we add the management of the <strong>CUSTOMER</strong>
* related analysis: remove the corresponding code if not necessary.
*/
public class ServiceAnalyzer extends GeneralAnalyzer {

  	/** Number of times the input data did not match any customer in the database.						*/
	protected 				int 		no_customer 		= 0 ;

	/**
	* Standard constructor, creating the <code>ArrayList</code> object used to store the result
	* of each call when the web service is deployed in a multi-threading environment (default).
	*/
	public ServiceAnalyzer() {

		this.list = new ArrayList(100) ;
	}

// **************************************************************************************************************
// **********************************			ANALYSIS			*********************************************
// **************************************************************************************************************

	/**
	* Produce a <code>StringBuffer</code> object containing all the analysis data already formatted and
	* ready for printing.<br/>
	*
	* @return a string buffer object containing the formatted results of the analysis
	*/
	public StringBuffer getResults() {

		// Counting the number of queries executed
		// Notice that the no_customer case is already counted by the no_record case
		long total = sql_query + no_record + sql_error + dao_error ;

		// Getting the total execution time of the calls completed successfully
		int sum = 0 ;
		Object[] times = this.list.toArray() ;
		for (int i = 0 ; i < times.length ; i++) sum += ((Integer) times[i]).intValue() ;

		// Getting the number of calls without errors
		int complete = calls - (no_db + macro_error + no_customer) ;

		// Getting the number of times we begun accessing the macro level
		int macro = calls - no_db ;

		// Formatting analysis results
		StringBuffer report = new StringBuffer() ;
		report.append("Total calls to the service    : " + calls		+ "\n") ;
		report.append("Number of calls still hung up : " + table.size()	+ "\n") ;
		report.append("Time of first service call    : " + startup		+ "\n") ;
		report.append("\n") ;
		report.append("Calls interrupted by a macro error                   : " + 	macro_error 		+ "\n") ;
		report.append("Calls to the service that provided no result         : " + 	no_result 			+ "\n") ;
		report.append("Calls stopped because the database was missing       : " + 	no_db 				+ "\n") ;
		report.append("Calls interrupted because the customer was not found : " +	no_customer			+ "\n") ;
		report.append("Calls interrupted by a logic error                   : " + 	logic_error 		+ "\n") ;
		report.append("Calls completed with a validation error              : " + 	validation_error	+ "\n") ;
		report.append("Calls completed returning valid data                 : " + 	no_error 			+ "\n") ;
		report.append("Calls due to a command code (excluding current)      : " + 	command	 			+ "\n") ;
		report.append("\n") ;
		if (complete > 0) {
			report.append("Execution completed at macro level  : " + complete		+ "\n") ;
			report.append("Number of tries at the macro level  : " + macro 			+ "\n") ;
			report.append("Number of time measurements         : " + times.length 	+ "\n") ;
			report.append("Total execution time                : " + sum 			+ " ms [" + (sum/complete) + " ms]\n") ;
			report.append("Total SQL queries                   : " + total 			+ " [" + (total/macro) + "]\n") ;
			report.append("\n") ;
		}
		if (total > 0) {
			report.append("SQL queries without problems        : " + sql_query + " [" + (100*sql_query/total) + "%]\n") ;
			report.append("SQL queries with no result          : " + no_record + " [" + (100*no_record/total) + "%]\n") ;
			report.append("SQL statement in error state        : " + sql_error + " [" + (100*sql_error/total) + "%]\n") ;
			report.append("DBA unexpected errors (not handled) : " + dao_error + "\n") ;
			report.append("\n") ;
		}
		return report ;
	}

// **************************************************************************************************************
// **********************************			CHRONWATCH			*********************************************
// **************************************************************************************************************

	/**
	* Stop the chronwatch after the end of one call service, even if the call generated an error.
	* If the chronwatch is stopped after an execution error, the error code is identified using the static
	* values defined in the <code>ErrorMessage</code> class.
	*
	* @param code the integer code describing the event or error
	* @see ErrorMessage
	*/
	public void endCall(int code) {

		// 1. Incrementing counters
		switch(code) {
			case ErrorMessage.MACRO				: addMacroError() ;			break ;
			case ErrorMessage.LOGIC				: addLogicError() ;			break ;
			case ErrorMessage.COMMAND			: addCommand() ;			break ;
			case ErrorMessage.NO_DB 			: addNoDatabase() ;			break ;
			case ErrorMessage.VALIDATION		: addValidationError() ;	break ;
			case ErrorMessage.OK				: addNoError() ;			break ;
			case ErrorMessage.NO_RESULT			: addNoResult() ;			break ;
			case ErrorMessage.NO_CUSTOMER		: addNoCustomer() ;			break ;
		}

		// 2. Finding the starting time for the current thread
		Thread current 	= Thread.currentThread() ;
		Long start		= (Long) this.table.get(current) ;

		// 3. Stopping chronwatch only if actually started
		if (start != null) {
			// Evaluating elapsed time
			int time = (int) (System.currentTimeMillis() - start.longValue()) ;
			// Adding the time measure only if the call was successfully
			if (code == ErrorMessage.OK) this.list.add(time) ;
			if (Settings.VERBOSE) {
				System.out.println(leadingName + "Call execution time  : " + time + " ms") ;
				System.out.println(leadingName + "Call execution thread: " + current) ;
			}
			// Remembering that we stopped the chronwatch
			this.table.remove(current) ;
		// 4. In the thread is not in the hash table, something went wrong
		} else {
			logger.logWarning(leadingName + "No starting time for thread " + current) ;
		}
	}

// **************************************************************************************************************
// **********************************			MUTATORS		*************************************************
// **************************************************************************************************************

	/**
	* Mutator method.
	*/
	public void addNoCustomer() 			{	this.no_customer++ ;										}

// **************************************************************************************************************
// **********************************			ACCESSORS		*************************************************
// **************************************************************************************************************

	/**
	* Accessor method.
	*/
	public int getNoCustomer() 				{	return this.no_customer ;									}

} // End of class