/**
* Title: 		ServiceData
* Description:	The ValueObject class wrapping the main business logic parameters
* Copyright:    Copyright (c) 2009-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;

import java.io.* ;
import java.util.* ;

/**
* The container for all STATIC settings used by the business service logic (string values etc.).<br/>
* Instantiation of this class should not be necessary since these methods are called only once, within a static
* code block of the <code>ServiceMacro</code> class, in order to retrieve all the static settings used by the
* business logic. All these settings are stored into static attributes of this class, hence these parameters
* are available without the need to instantiate this class.
*/
public class ServiceData {

	/** List of the keywords used by this service (loaded from the property file).							*/
	protected static		Hashtable<String, String> 	hash ;
   	/** List of the keywords used by this service (loaded from the property file).							*/
	protected static 		StringBuffer[]				data_keys ;

	/** If <code>true</code> the <code>ServiceMacro</code> will append two fake records to each call.		*/
	public static 			boolean 					FAKE 			= true ;		// Default value
	/** Time (in milliseconds) used to sleep the HTTP-request thread when testing concurrent call.			*/
	public static 			int							DELAY 			= 0 ;			// Default value
	/** If <code>true</code> the <code>ServiceLogic</code> requires the some authentication check.			*/
	public static 			boolean 					HEADER_CHECK		= true;		// Default value
	/**	The value of the authentication parameter used to check the client request 							*/
	public static 			String						HEADER_VALUE ;

   	/** Beginning of each logging message for this class.													*/
	protected static 		String 						leadingName 	= "ServiceData - " ;

	/**
	* Load the logical settings for the business logic from the <code>configuration.properties</code> file and
	* store them into this class attributes. The actual name of the configuration file is defined by the
	* <code>Settings.CONFIG</code> parameter.
	*
	* @param the name of the properties file (usually is "configuration.properties")
	* @return <code>true</code> if all the logic flags have been loaded successfully
    */
    public static boolean loadLogicFlags(String source) {

		// 1. Loading logical flags from the properties file
		try {
			ResourceBundle rb	= ResourceBundle.getBundle(source) ;

			// Retrieving boolean properties
			String fakeData		= rb.getString("logic.fake") ;
			String headerCheck	= rb.getString("logic.check_header") ;
			FAKE				= fakeData.equalsIgnoreCase("YES") ;
			HEADER_CHECK		= headerCheck.equalsIgnoreCase("YES") ;

			// Retrieving string properties
			HEADER_VALUE		= rb.getString("header.value") ;

			// Parsing integer properties
			String value		= rb.getString("logic.delay") ;
			try 				{ DELAY		= Integer.parseInt(value.trim()) ;		}
			catch(Exception e) 	{ DELAY 	= 0 ;									}	// Default value


		} catch(MissingResourceException e) {
			System.err.println(leadingName + "Configuration file not found: using defaults") ;
			e.printStackTrace() ;
			return false ;
		}

		// 2. Eventually showing logical settings on the STDOUT
		if (Settings.DEBUG) {
			String blank = "none " ;
			System.out.print(leadingName + "Logical flags: ") ;
			System.out.print(FAKE?"FAKE ":blank) ;
			System.out.print(HEADER_CHECK?"HEADER_CHECK":blank) ;
			System.out.println() ;
		}

		return true ;
	}

	/**
	* Load all the keywords from the <code>keywords.properties</code> file and store them into the
	* <code>data_keys</code> class attribute. The actual name of the configuration file is defined by the
	* <code>Settings.KEYWORDS</code> parameter.
	*
	* @param the name of the properties file (usually is "keywords.properties")
	* @return <code>true</code> if the keywords have been successfully loaded from the properties file
    */
    public static boolean loadKeywords(String source) {

		boolean fromFile ;
		String 	value 	= null ;
				hash 	= new Hashtable<String, String>() ;
		// Deprecated
		//Vector v = new Vector(100) ;

		// 1. Loading all values from the properties file
		try {
			ResourceBundle 	rb 	= ResourceBundle.getBundle(source) ;
			Enumeration 	e 	= rb.getKeys() ;
			while ( e.hasMoreElements() ) {
				String key = (String) e.nextElement() ;
				value = rb.getString(key) ;
				hash.put(key, value) ;
				// Deprecated
				//v.add( new StringBuffer(value) ) ;
			}
			fromFile = true ;
			if (Settings.VERBOSE) System.out.println(leadingName + "Loaded " + hash.size() + " values from: " + source + ".properties") ;

		} catch(MissingResourceException mre) {
			fromFile = false ;
			System.err.println(leadingName + "Service data file not found") ;
			mre.printStackTrace() ;
		}

		// 2. If successful loads the keywords into this class's array
		if (fromFile) {
			// Deprecated:
			// Se necessario settare la lista delle keywords ANCHE nell'array data_keys, prendendo i dati
			// dalla hastable ... ma forse non e' piu' necessario: al momento sembra bastare che le keywords
			// siano nella hash
			//data_keys = (StringBuffer[]) v.toArray( new StringBuffer[v.size()] ) ;
			//if (Settings.VERBOSE) System.out.println(leadingName + "Loaded " + data_keys.length + " keywords") ;
		}	else {
			// 2.a Otherwise we use the hard-coded values (see below)
			data_keys = getHardCodedValues() ;
			if (Settings.VERBOSE) System.out.println(leadingName + "Using " + data_keys.length + " hardcoded keywords") ;
		}

		// Keep for testing or debug
		//printHashtable(hash) ;		System.exit(0) ;

		// 3. Returning result
		return fromFile ;
	}

// *************************************************************************************************************
// ***************************					ACCESSORS				****************************************
// *************************************************************************************************************

	/**
	* Accessor method.
	*/
	public static Hashtable getHashtable()				{		return hash ;									}

	/**
	* Accessor method.
	*
	* @deprecated
	*/
	public static StringBuffer[] getKeywords()			{		return data_keys ;								}

// *************************************************************************************************************
// ***************************			COMMAND LINE UTILITIES				************************************
// *************************************************************************************************************

	/**
	* Create a <code>keywords.properties</code> data file containing the list of all the necessary keywords.
	* The input source file can be specified running the <code>SimpleClient</code> from the command line,
	* using the option <code>-c <em>file name</em></code>. If no path it's specified, the source file
	* will be searched in the current directory. Eventually it's possible to specify an absolute path,
	* for example: <code>C:/Develop/name_list.txt</code>.
	*
	* @param source the source file name
	* @return true if the <code>keywords.properties</code> has been produced successfully
    */
    public static boolean createKeywords(String source) {

		// 1. Initializing
		BufferedReader reader = null ;
		BufferedWriter writer = null ;
		String sink = "keywords.properties" ;
		System.out.println() ;

		// 2. Opening input stream using a BufferedReader
		try {
			FileReader fileReader = new FileReader(source) ;
			reader = new BufferedReader(fileReader) ;
			System.out.println(leadingName + "Opened file: " + source) ;
        } catch (FileNotFoundException filenotfound) {
            System.err.println(leadingName + "Could not open file: " + source) ;
			return false ;
        }

		// 3. Reading and writing data into a BufferedWriter
		int i = 0 ;
		String line = null ;
		try {
            FileWriter fileWriter = new FileWriter(sink) ;
			writer = new BufferedWriter(fileWriter) ;
			// Main cycle running the input file
			while ( (line = reader.readLine()) != null ) {
				line = i++ + "=" + line ;
				writer.write(line, 0, line.length()) ;
				writer.newLine() ;
			}
			reader.close() ;
			writer.close() ;
        } catch (IOException exception) {
            System.err.println(leadingName + "Error reading file") ;
			return false ;
        }

		// 4. Logging result and returning
		if (Settings.VERBOSE) System.out.println(leadingName + "Created data file: " + sink) ;
		return true ;
	}

	/**
	* Create a <code>keywords.java</code> data file containing the Java source definition of all the keywords.
	* This method should be used only during development, when the properties file should be always available,
	* to produce the hard-coded list that will be retrieved by the <code>getHardCodedValues()</code> method.
	*
	* @return true if the <code>keywords.java</code> has been produced successfully
	* @see getHardCodedValues()
    */
    public static boolean produceJavaFile() {

		// 1. Initializing
		String sink = "keywords.java" ;
		BufferedWriter writer = null ;

		if (data_keys == null) {
			System.err.println(leadingName + "Keywords not loaded") ;
			return false ;
		}

		try {
			int howMany = data_keys.length ;
            FileWriter fileWriter = new FileWriter(sink) ;
			writer = new BufferedWriter(fileWriter) ;
			for (int i = 0; i < howMany; i++) {
				writer.write("buffer[" + i + "] = new StringBuffer(\"" + data_keys[i] + "\") ;\n") ;
			}
			writer.close() ;
        } catch (IOException exception) {
            System.err.println(leadingName + "Error reading file") ;
			return false ;
        }

		if (Settings.VERBOSE) System.out.println(leadingName + "Produced Java file: " + sink) ;
		return true ;
	}

	/**
	* Prints on the STDOUT the list of all the keywords loaded by this class.
	* This method should be used only during development, for testing or debug.
	*
	* @return true if the keywords list has been printed succesfully
    */
    public static boolean printKeywords() {

		if (data_keys == null) return false ;
		int howMany = data_keys.length ;

		System.out.println(leadingName + "KEYWORD LIST:\n") ;
		for (int i = 0; i < howMany; i++) System.out.println("buffer[" + i + "] = \"" + data_keys[i] + "\"") ;
		System.out.println() ;

		return true ;
	}

// *************************************************************************************************************
// ***************************				HARD-CODED VALUES				************************************
// *************************************************************************************************************

	/**
	* Return an hard-coded list of all the necessary keywords.
	*
	* @return the <code>StringBuffer</code> array containing all the required keywords
	* @see produceJavaFile()
    */
    public static StringBuffer[] getHardCodedValues() {

		StringBuffer[] buffer = new StringBuffer[11] ;

		buffer[0]  = new StringBuffer("Axis2"		) ;
		buffer[1]  = new StringBuffer("Service"		) ;
		buffer[2]  = new StringBuffer("Web"			) ;
		buffer[3]  = new StringBuffer("Bug fixing"	) ;
		buffer[4]  = new StringBuffer("Java"		) ;
		buffer[5]  = new StringBuffer("Run"			) ;
		buffer[6]  = new StringBuffer("Deploy"		) ;
		buffer[7]  = new StringBuffer("Develop"		) ;
		buffer[8]  = new StringBuffer("Tomcat"		) ;
		buffer[9]  = new StringBuffer("Publish"		) ;
		buffer[10] = new StringBuffer("Test"		) ;

		return buffer ;
	}

 // *************************************************************************************************************
 // ***************************				TEST OR DEBUG					************************************
 // *************************************************************************************************************

 	/**
 	* Prints on the STDOUT the list of all the keywords loaded by this class.
 	* This method should be used only during development, for testing or debug.
 	*
 	* @param hash an hash table, usually one of those produced by the <code>loadKeywords()</code> method
     */
     public static void printHashtable(Hashtable<String, String> hash) {

 		int counter = 0 ;
 		Enumeration e = hash.keys() ;

 		System.out.println() ;
 		while ( e.hasMoreElements() ) {
 			counter++ ;
 			String key 		= (String) e.nextElement() ;
 			String value 	= hash.get(key) ;
 			System.out.println(leadingName + counter + " : " + key + " = " + value) ;
 		}
 	}

} // End of class