/**
* Title: 		ServiceLogic
* Description:	The business logic layer of the web service
* Copyright:    Copyright (c) 2009-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.service ;

import adriani.jws.template.* ;
import adriani.jws.template.db.* ;
import adriani.jws.template.util.* ;
import adriani.jws.template.wrappers.*;
import adriani.jws.template.wsdl.*;

import java.util.* ;

/**
* The main business logic class, implementing the high level behavior of the service. All the issues
* concerning the preparation of the parameters for the DAO layer should not be handled here, but should be
* delegated to the <code>ServiceMacro</code> instead.<br/>
* Using this class it should be possible to implement the service's skeleton just adding a code-block
* like this:<br/><br/>
*
* <pre><code>
*	ServiceLogic logic      	= new ServiceLogic() ;
*	WrapperIn input             = new WrapperIn(adbBean) ;
*	WrapperDataOut output       = (WrapperXXXOut) logic.getOutput(input) ;
*	ADBBean element             = output.getElement() ;
* </code></pre><br/><br/>
*
* Hence this class should implement a <code>getOutputXXX()</code> method for each method offered by the
* service skeleton. Every <code>getOutputXXX()</code> method should return the proper
* <code>WrapperXXXOut</code>, i.e. a custom subclass of the <code>WrapperOut</code> class.
*/
public class ServiceLogic extends GeneralService {

  	/** A reference to the <code>ServiceMacro</code> managing all the calls the necessary DAO's.			*/
	protected 					ServiceMacro 	serviceMacro ;

  	/** The header value as retrieved from the SOAP's request (depending by the project).					*/
	protected 					String		 	header_value ;

  	/** Static value identifying the 1st web service method.												*/
	public static final			int  			GET_INFORMATION		= 1 ;
  	/** Static value identifying a request of the 2nd web service method (PLACE-HOLDER).					*/
	//protected static final	int				SECOND			= 2 ;

  	/** The static value used to identify the INFO empty response.											*/
	protected static final		int  			SILENT 			= 1 ;
  	/** The static value used to identify the INFO empty response.											*/
	protected static final		int  			INFO 			= 2 ;
	/** The static value used to identify the WARNING empty response.										*/
	protected static final		int  			WARN 			= 3 ;
	/** The static value used to identify the ERROR empty response.											*/
	protected static final		int  			ERROR 			= 4 ;

  	/** The hard-coded pattern used to recognize remote manager commands (i.e. "000000").					*/
	protected static final		String 			command_pwd		= "000000" ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static			String 			leadingName 	= "ServiceLogic - " ;

	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 			WSLogger		logger 			= WSLogger.getLogger(ServiceLogic.class) ;

	/**
	* Constructor, creating also the <code>ServiceMacro</code> object and storing it as class attribute.
    */
    public ServiceLogic() {

		this.serviceMacro = new ServiceMacro() ;
	}

	/**
	* Constructor, creating also the <code>ServiceMacro</code> object and storing it as class attribute.
	* This constructor should be used when we need to retrieve (and store here) some data from the
	* SOAP's header.
	*
	* @input value the value retrieved from the SOAP's header
    */
    public ServiceLogic(String value) {

		this() ;
		this.header_value = value ;
	}

	/**
	* Constructor, creating also the <code>ServiceMacro</code> object and storing it as class attribute.
	* <br/><br/>
	*
	* NOT USED AT THE MOMENT!
	*
	* @param macro the <code>ServiceMacro</code> it's instantiated only if this parameter is <code>true</code>
    */
    public ServiceLogic(boolean macro) {

		if (macro) this.serviceMacro = new ServiceMacro() ;
	}

 // *************************************************************************************************************
 // **********************************			COMMON API				****************************************
 // *************************************************************************************************************

    /**
    * Checking for authentication, comparing authentication from the SOAP's header with authentication data from
    * the properties file.
    */
    public WrapperOut checkAuthentication(int method) {

 		// Checking authentication consistency
 		if (ServiceData.HEADER_CHECK) {
 			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Checking authentication: " + this.header_value) ;
 			if ( (this.header_value != null) && (ServiceData.HEADER_VALUE != null) ) {
 				if ( ServiceData.HEADER_VALUE.equalsIgnoreCase(this.header_value) )	{
 					logger.logInfo(leadingName + "Perfect authentication match") ;
 				} else {
 					logger.logSevere(leadingName + "Wrong authentication") ;
 					return getEmptyResponse(method, SILENT, ErrorMessage.BAD_DATA) ;
 				}
 			} else {
 				logger.logSevere(leadingName + "Missing authentication data: " + this.header_value + "-" + ServiceData.HEADER_VALUE) ;
 				return getEmptyResponse(method, SILENT, ErrorMessage.BAD_DATA) ;
 			}
 			//System.out.println(leadingName + "Compared authentication keys: " + this.header_value + "-" + ServiceData.HEADER_VALUE) ;
 		} else {
 			logger.logWarning(leadingName + "Skipping check on the pre-shared key") ;
 		}
 		return null ;
     }

	/**
	* General purpose service layer for all the web service's methods. When the service is invoked the <code>invokeBusinessLogic()</code>
	* method of the <code>XXXMessageReceiverInOut</code> class forwards the action to
	* <code>XXXSkeleton.operation()</code>, which can call this method.<br/><br/>
	*
	* This method can be used by all methods using a "standard pattern", by calling:<br/><br/>
	* 
	* <pre><code>
	* 
	* </code></pre>
	*
	* @param input the input <code>ADBBean</code> wrapped by the <code>WrapperIn</code> class
	* @return the full output data wrapped by the proper <code>WrapperOut</code> class
    */
    public WrapperOut invokeMethod(int method, WrapperIn input) {
    	
		// 1. Starting call
		ArrayList list = null ;
		if (Settings.ANALYZE) serviceAnalyzer.startCall() ;
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Input wrapper: " + input) ;

		// 2. Eventually managing the web service
		if (method == GET_INFORMATION) {
			if ( Settings.MANAGER && manageService(input) )
			return getEmptyResponse(method, SILENT, ErrorMessage.COMMAND) ;
		}
		
		// 3. Checking the Pre-Shared Key consistency
		WrapperOut authentication = checkAuthentication(method) ;
		if (authentication != null) return authentication ;
    	
		// 4. Adjusting input data
		try {
			if (!input.validate()) {
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Invalid input request " + input) ;
				return getEmptyResponse(method, INFO, ErrorMessage.NOK) ;
			}
			if (Settings.VERBOSE) System.out.println(leadingName + "Valid input: " + input) ;
		} catch(Exception e) {	
			return getEmptyResponse(method, INFO, ErrorMessage.NOK) ;	
		}

		// 5.1 First case: retrieving data using a fake query (does not require any actual database connection)
		if (ServiceData.FAKE) {
			try {
				switch(method) {
					case 	GET_INFORMATION	: list = serviceMacro.getDummyInformation	( (WrapperInformationIn)	input ) ;	break ;
					//case 	SECOND			: list = serviceMacro.getDummyInformation	( (WrapperInformationIn)	input ) ;	break ;
					default 				: list = serviceMacro.getDummyInformation	( (WrapperInformationIn)	input ) ;
				}
			} catch(Exception e) {	return handleMacroError(method, e, true) ;	}
		// 5.2 Second case: retrieving data using actual DAO queries (requires connection to the database)	
		} else {
			try {
				switch(method) {
					case 	GET_INFORMATION	: list = serviceMacro.getInformation		( (WrapperInformationIn)	input ) ;	break ;					
					//case 	SECOND			: list = serviceMacro.getInformation		( (WrapperInformationIn)	input ) ;	break ;					
					default					: list = serviceMacro.getInformation		( (WrapperInformationIn)	input ) ;								
				}
			} catch(Exception e) {	return handleMacroError(method, e, true) ;	}
		}
		//System.out.println(leadingName + "Fetched data: " + list) ;

		// 6. Handling critical cases (list == null)
		int  status =  serviceMacro.getStatus() ;
		if ( status == serviceMacro.NO_DATABASE	) 		return getEmptyResponse(method, SILENT, ErrorMessage.NO_DB			) ;
		if ( status == serviceMacro.NO_CUSTOMER	) 		return getEmptyResponse(method, SILENT, ErrorMessage.NO_CUSTOMER	) ;
		//System.out.println(leadingName + "No critical errors") ;

		// 7. If necessary sleeping current thread to easy concurrency test or debug
		if (ServiceData.DELAY > 0) {
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Stopping current thread for " + ServiceData.DELAY + " ms") ;
			delay(ServiceData.DELAY) ; 
		}
		if (Settings.VERBOSE) System.out.println(leadingName + "Retrieved list: " + list) ;
		
		// 8. Creating outputinvokeBusinessLogic(int method, WrapperIn input) { element
		WrapperOut output = null ;
		if (list != null && list.size() > 0) {
			// 8.1 Creating proper WrapperOut object
			switch(method) {
				case 	GET_INFORMATION		: 	WrapperInformation[] record = (WrapperInformation[]) list.toArray( new WrapperInformation[list.size()] ) ;
											output = new WrapperInformationOut(record) ;
											break ;
			}
			if (!ServiceData.FAKE) {
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Retrieved data from database"		) ;
			} else {
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Retrieved hard-coded fake data"	) ;
			}
			//System.out.println(leadingName + "Set list size: " + list.size()) ;
		} else {
			// 8.2 We should already handled the other cases "list == null" at step 6
			switch(method) {
				case 	GET_INFORMATION		: return getEmptyResponse(method	, INFO	, ErrorMessage.NO_RESULT	) ;
				default					: return getEmptyResponse(method	, INFO	, ErrorMessage.NO_RESULT	) ;
			}
		}
		
		// Keep for testing or debug (forces a LOGIC ERROR case)
		//output = null ;
		
		// 9 Validating output element (and all his inner elements)
		if (output != null) {
			// 9.1 Handling successful operation
			if (status == ServiceMacro.OK) {
				output.setStatus	( WrapperOut.STATUS_OK		) ;
				output.setMessage	( WrapperOut.MESSAGE_OK		) ;
				// Setting custom wrapper data: necessary only if we have data below the root element of the response
				/*switch(method) {
					case 	GET_INFORMATION	: 	((WrapperInformationOut) output).setValue( serviceMacro.getValue() ) ;		break ;
				}*/
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Operation succesful (" + method + ")") ;
			// 9.2 Handling failure				
			} else {
				output.setStatus	( WrapperOut.STATUS_WRONG 	) ;
				output.setMessage	( WrapperOut.MESSAGE_NOK 	) ;
				output.reset() ;
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Operation failed (" + method + ")") ;
			}
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Validating output data"	) ;
			try 						{	output.validate() ;							}
			catch(WrapperFault e) 		{	return handleNoValidation(output, e) ;		}
			// PLACEHOLDER: could be necessary, depending by the project
			//catch(WrapperOutFault e) 	{	return handleNoValidation(output, e) ;							}
		}
		// 9.3 If we reach this block there's something wrong in the above logic
		if (output == null) {
			return getEmptyResponse(method, ERROR, ErrorMessage.LOGIC	) ;
		}
		//System.out.println(leadingName + "Produced output: " + output) ;

		// Keep for testing or debug (forces an unexpected case: not handled by design)
		//Object foo = null ;		foo.toString() ;

		// 10. Eventually stopping the chronwatch.
		if (Settings.ANALYZE) serviceAnalyzer.endCall(ErrorMessage.OK) ;

		// 11. Returning result
		return output ;
    }
    
// *************************************************************************************************************
// **********************************			BUSINESS LOGIC			****************************************
// *************************************************************************************************************

	/**
	* This is main business logic for the web service. When the service is invoked the <code>invokeBusinessLogic()</code>
	* method of the <code>XXXMessageReceiverInOut</code> class forwards the action to
	* <code>XXXSkeleton.operation()</code>, which should just calls this method.<br/>
	*
	* @param input the input <code>ADBBean</code> wrapped by the <code>WrapperInformationIn</code> class
	* @return the full output data wrapped by the <code>WrapperInformationOut</code> class
    */
    public WrapperInformationOut getInformation(WrapperInformationIn input) {
    	
    	return (WrapperInformationOut) invokeMethod(GET_INFORMATION, input) ;
    }
    
// *************************************************************************************************************
// **********************************		OLD BUSINESS LOGIC			****************************************
// *************************************************************************************************************

	/**
	* Business logic for the first operation offered by the web service (currently the only one).<br/>
	* When the web service is invoked the <code>invokeBusinessLogic()</code> method of the
	* <code>XXXMessageReceiverInOut</code> class forwards the action to
	* <code>XXXSkeleton.operation()</code>, which should just calls this method.<br/>
	*
	* @param input the input <code>ADBBean</code> element wrapped by the <code>WrapperIn</code> class
	* @return the full output data wrapped by the <code>WrapperDataOut</code> class
	* @see TemplateSkeleton#getInformation(User user)
	* @deprecated: use <code>getOutput(WrapperInformationIn input)</code> instead
	* @see #getOutput(WrapperInformationIn input)
    */
    public WrapperInformationOut plainGetOutput(WrapperInformationIn input) {

		// 1. Starting call
		ArrayList list = null ;
		if (Settings.ANALYZE) serviceAnalyzer.startCall() ;
		if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Input wrapper: " + input) ;

		// 2. Eventually managing the web service
		if ( Settings.MANAGER && manageService(input) )
		return (WrapperInformationOut) getEmptyResponse(GET_INFORMATION, SILENT, ErrorMessage.COMMAND) ;

		// 3. Checking authentication data
		Object authentication = checkAuthentication(GET_INFORMATION) ;
		if (authentication != null) return (WrapperInformationOut) authentication ;

		// 4. Adjusting input data
		if (!input.validate()) {
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Invalid input request " + input) ;
			return (WrapperInformationOut) getEmptyResponse(GET_INFORMATION, INFO, ErrorMessage.NOK) ;
		}
		if (Settings.VERBOSE) System.out.println(leadingName + "Valid input: " + input) ;

		// 5. Retrieving data from the database
		if (ServiceData.FAKE) {
			try 					{ 	list = serviceMacro.getDummyInformation(input) ;						}
			catch(Exception e)		{	return (WrapperInformationOut) handleMacroError(GET_INFORMATION, e, true) ;	}
		} else {
			try 					{ 	list = serviceMacro.getInformation(input) ;								}
			catch(Exception e)		{	return (WrapperInformationOut) handleMacroError(GET_INFORMATION, e, true) ;	}
		}
		//System.out.println(leadingName + "Fetched data: " + list) ;

		// 6. Handling critical cases (list == null)
		int  status =  serviceMacro.getStatus() ;
		if ( status == serviceMacro.NO_DATABASE	) 	return (WrapperInformationOut) getEmptyResponse(GET_INFORMATION, SILENT, ErrorMessage.NO_DB		) ;
		if ( status == serviceMacro.NO_CUSTOMER	) 	return (WrapperInformationOut) getEmptyResponse(GET_INFORMATION, SILENT, ErrorMessage.NO_CUSTOMER	) ;
		//System.out.println(leadingName + "No critical errors") ;

		// 7. Eventually sleeping current thread to easy concurrency test or debug
		if (ServiceData.DELAY > 0) {
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Stopping current thread for " + ServiceData.DELAY + " ms") ;
			delay(ServiceData.DELAY) ;
		}
		if (Settings.VERBOSE) System.out.println(leadingName + "Retrieved list: " + list) ;

		// Keep for testing or debug (forces a NO RESULT case)
		//list = null ;

		// 8. Creating output element
		WrapperInformation[] 	record = null ;
		WrapperInformationOut 	output = null ;
		if (list != null) {
			record = (WrapperInformation[]) list.toArray( new WrapperInformation[list.size()] ) ;
			output = new WrapperInformationOut(record) ;
			if (!ServiceData.FAKE) {
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Retrieved data from database"		) ;
			} else {
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Retrieved hard-coded fake data"	) ;
			}
			//System.out.println(leadingName + "Set list size: " + list.size()) ;
		} else {
			// 8.a We handled the other cases "list == null" at step 5
			return (WrapperInformationOut) getEmptyResponse(GET_INFORMATION, INFO, ErrorMessage.NO_RESULT) ;
		}

		// Keep for testing or debug (forces a LOGIC ERROR case)
		//output = null ;

		// 9. Validating output element (and all his inner elements)
		if (output != null) {
			// 9.a Checking logic condition
			if (status == ServiceMacro.OK) {
				output.setStatus	( WrapperOut.STATUS_OK		) ;
				output.setMessage	( WrapperOut.MESSAGE_OK		) ;
				//output.setValue	( serviceMacro.getValue()	) ;
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Found customer"		) ;
			} else {
				output.setStatus	( WrapperOut.STATUS_WRONG 	) ;
				output.setMessage	( WrapperOut.MESSAGE_NOK 		) ;
				output.reset() ;
				if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Customer not found"	) ;
			}
			if (logger.isInfoEnabled()) logger.logInfo(leadingName + "Validating output data"	) ;
			try 						{	output.validate() ;												}
			//catch(WrapperFault e) 		{	return (WrapperDataOut) handleNoValidation(output, e) ;			}
			catch(WrapperFaultOut e) 	{	return (WrapperInformationOut) handleNoValidation(output, e) ;			}
		}
		// 9.b If we reach this block there's something wrong in the above logic
		if (output == null) {
			return (WrapperInformationOut) getEmptyResponse(GET_INFORMATION, ERROR, ErrorMessage.LOGIC) ;
		}
		//System.out.println(leadingName + "Produced output: " + output) ;

		// Keep for testing or debug (forces an unexpected case: not handled by design)
		//Object foo = null ;		foo.toString() ;

		// 10. Eventually stopping the chronwatch.
		if (Settings.ANALYZE) serviceAnalyzer.endCall(ErrorMessage.OK) ;

		// 11. Returning result
		return output ;
	}

// *************************************************************************************************************
// **********************************		WRAPPERS FACTORY		********************************************
// *************************************************************************************************************

	/**
	* The <strong>factory</strong> creating the proper <code>WrapperXXXOut</code> object that will be returned
	* by the corresponding <code>getOutputXXX()</code> method. Every possible <code>WrapperXXXOut</code> class
	* must extend the parent <code>WrapperOut</code> class.<br/>
	* This factory should return the proper <code>WrapperXXXOut</code> object corresponding to the input
	* <em>method</em> parameter, which describes one of the operations offered by the skeleton
	* (listed using static integer values defined as class fields).<br/><br/>
	*
	* In current implementation this factory is accessed only the the <code>getEmptyResponse()</code>
	* method.
	*
	* @param method the logic method that invoked this method (see class fields)
	* @return the proper <code>WrapperOut</code> object for the corresponding logic method
    */
    public static WrapperOut getWrapperOut(int method) {

		// 1. Preparing the proper output object
		WrapperOut wrapper = null ;
		switch(method) {
			case	GET_INFORMATION	:	wrapper = new WrapperInformationOut() ;		break ;
			//case	SECOND			:	wrapper = new WrapperXXXOut() ;				break ;
		}

		// 2. Eventually logging creation failure
		if (wrapper == null) {
			logger.logSevere(leadingName + "WrapperOut factory failed for method " + method) ;
		}

		return wrapper ;
	}

// *************************************************************************************************************
// **********************************		ERRORS MANAGEMENT		********************************************
// *************************************************************************************************************

	/**
	* Prepare a valid <code>WrapperOut</code> subclass when the exception comes from a lower level, like the
	* <code>ServiceMacro</code> class or the <code>ServiceDBA</code> class. In this case the error message is
	* obtained by reading the necessary <code>StackTraceElement</code> objects.<br/><br/>
	*
	* Since the macro processing has been interrupted before producing any SOAP output, here we produce
	* a brand new <code>WrapperOut</code> wrapper to ensure a correct SOAP output response, containing
	* a message like "Java Macro Error".
	*
	* @param method the logic method that invoked this method (see class fields)
	* @param exception the <code>Exception</code> thrown from a lower level (<code>null</code> otherwise)
	* @param last if <code>true</code> the <code>getEmptyResponse()</code> method is invoked
	* @return a valid <code>WrapperOut</code> subclass with a message describing this situation
    */
    protected WrapperOut handleMacroError(int method, Exception exception, boolean last) {

		// 1. Closing connections to databases (we don't know if they have been left opened)
    	// Usually they should have already been closed by the ServiceMacro.handleNoDatabase() method
    	if (Settings.DEBUG) System.out.println(leadingName + "Handling macro error for method 1") ;
		ServiceDAO serviceDAO = serviceMacro.getServiceDAO() ;
		if (serviceDAO != null) {
			serviceDAO.closeConnection(Database.DB_01) ;
			serviceDAO.closeConnection(Database.DB_02) ;
			logger.logWarning(leadingName + "Forced connections check in for all databases") ;
		}

		// 2. Retrieving exception message
		String error = null ;
		if (exception != null) try {
			exception.printStackTrace() ;
			StackTraceElement[] stackTrace = exception.getStackTrace() ;
			error = stackTrace[0].getClassName() + "." + stackTrace[0].getMethodName() + " " + exception ;
		}
		catch(Exception e) {
			e.printStackTrace() ;
			error = "STACKTRACE ERROR (" + e.getMessage() + ") for " + exception.getMessage() ;
		}

		// 3. Logging error message
		String message = ErrorMessage.getMessage(ErrorMessage.MACRO) ;
		logger.logSevere(leadingName + message + ": " + error) ;

		// 4. If this is the last macro operation, we create the output wrapper in order to send a
		// complete SOAP message back to the client (the main logic will quit processing).
		if (last) 	return getEmptyResponse(method, SILENT, ErrorMessage.MACRO) ;
		else 		return null ;
	}

	/**
	* Prepare a valid <code>WrapperOut</code> subclass when we fail validation of the original
	* <code>WrapperOut</code> object produced by the main logic. The output message will contain a message
	* with a reference to all the <code>ADBBean</code> objects responsible for the validation error.
	*
	* @param wrapper the instance of the <code>WrapperOut</code> subclass wrapping the output data
	* @param exception the <code>Exception</code> thrown when validating the <code>WrapperOut</code> element
	* @return a valid <code>WrapperOut</code> subclass with a message describing all the validation errors
    */
    protected WrapperOut handleNoValidation(WrapperOut wrapper, Exception e) {

		// Logging only the validation error message due to the <code>Wrapper</code> inner wrappers,
		// because any other error (if any) has already been logged.
		String invalid = ErrorMessage.getMessage(ErrorMessage.VALIDATION) ;
		String message = invalid + ": " + e.getMessage() ;
		logger.logSevere(leadingName + message) ;

		// Forcing message in the wrapper class field and in the ADBBean element
		wrapper.forceStatus(WrapperOut.STATUS_WRONG) ;
		wrapper.forceMessage(message) ;

		// Eventually ending analysis
		if (Settings.ANALYZE) serviceAnalyzer.endCall(ErrorMessage.VALIDATION) ;
		return wrapper ;
	}

	/**
	* Prepare a valid <code>WrapperOut</code> object when it's not possible to produce a valid result.
	* This method can be called in many different situations, each situation corresponding to a different
	* message to be logged while producing the empty response. These situations are identified using the
	* <code>state</code> param, which should be one of the integer values mapped by the
	* <code>ErrorMessage</code> class. In current implementation this method is called in following
	* situations: COMMAND, NO_DB, NO_RESULT, ERROR_LOGIC, ERROR_MACRO.<br/><br/>
	*
	* The <code>level</code> parameter can be: SILENT, INFO, WARN or ERROR.<br/>
	* If the SILENT constant it's used when no log message must be produced here, because we assume the caller
	* has already logged a message describing the situation. Otherwise we produce a log message with
	* the specified level.
	* Current implementation relies on the wrapper factory, implemented by the <code>getWrapperOut()</code>,
	* to produce the proper <code>WrapperOut</code> method for the current method.
	*
	* @param method a reference to one the SOAP methods exposed in the WSDL (see class fields)
	* @param level the integer value defining the level of the logged message
	* @param state the code of the message as described by the <code>ErrorMessage</code> class
	* @return a valid <code>WrapperOut</code> object with a message describing this situation
	* @see #getWrapperOut(int method)
    */
    protected WrapperOut getEmptyResponse(int method, int level, int state) {

		// 1. Ending analysis and getting error message
		if (Settings.ANALYZE) serviceAnalyzer.endCall(state) ;
		String message = ErrorMessage.getMessage(state) ;

		// 2. Eventually logging the error message
		switch(level) {
			case 	SILENT	:	break ;
			case 	INFO	:	if (logger.isInfoEnabled()) logger.logInfo(leadingName + message) ;		break ;
			case 	WARN	:	logger.logWarning(leadingName + message) ;								break ;
			case 	ERROR	:	logger.logSevere(leadingName + message) ;								break ;
			default			:	if (logger.isInfoEnabled()) logger.logInfo(leadingName + "EMPTY RESPONSE") ;
		}

		// 3. Getting the proper wrapper
		WrapperOut wrapper = getWrapperOut(method) ;

		// 4. Returning an empty valid response
		try 				{ wrapper.validate() ;						 								}
		catch(Exception e)	{ logger.logWarning(leadingName + "Failed validation of empty response") ;	}
		wrapper.forceStatus(WrapperOut.STATUS_WRONG) ;
		if (message != null) wrapper.forceMessage(message) ;
		return wrapper ;
	}

 // **********************************************************************************************************
 // **********************************			ACCESSORS				*************************************
 // **********************************************************************************************************

 	/**
 	* Return the instance of the <code>serviceMacro</code> used by this class.
 	*
 	* @return a reference to the <code>serviceMacro</code> used by this class
 	*/
     public ServiceMacro getServiceMacro() 				{			return this.serviceMacro ;				}

// *************************************************************************************************************
// **********************************		SERVICE MANAGEMENT			****************************************
// *************************************************************************************************************

 	/**
 	* Allow to configure the service dynamically, without reloading the AAR package on the application server.
 	* To send a valid command the service must be invoked passing a parameter starting with a sequence of
 	* zeros followed by the command code. The number of zeros depend by the constrain defined in the WSDL
 	* for the input parameter.<br/>
 	* For example, if the input number parameter must be at least three digit long, then a valid command code
 	* could be something like "0001", which will be interpreted as the command <strong>1</strong>.<br/><br/>
 	* The actual pattern used to recognize a command is defined by the <code>command_pwd</code> class field,
 	* while the field containing the command is defined by the <code>WrapperSearchIn</code> class.
 	*
 	* @param input an instance of the <code>WrapperSearchIn</code> wrapper containing the input data
 	* @return <code>true</code> if the input number have been recognized as valid command code
 	* @see ServiceLogic#command_pwd
 	* @see WrapperSearchIn#getCommand()
 	*/
     private boolean manageService(WrapperIn input) {

 		// Retrieving input field: see WrapperInformationIn for the command field mapping
 		String 			leading = null ;
 		StringBuffer 	cmd 	= ( (WrapperInformationIn) input ).getCommand() ;
 		if (cmd == null) return false ;

 		// Checking if we have a valid command code
 		int minimal = command_pwd.length() ;
 		if (cmd.length() >= minimal) leading = cmd.substring(0, minimal) ; else return false ;
 		if ( !leading.equalsIgnoreCase(command_pwd) ) return false ;
 		String action = cmd.substring(minimal) ;

 		// Parsing input string
 		int key = 0 ;
 		try 				{ key = Integer.parseInt(action.trim()) ;	}
 		catch(Exception e) 	{ key = 0 ;									}

 		// Logging multilanguage message
 		String message = ErrorMessage.getMessage(ErrorMessage.COMMAND) ;
 		if (logger.isInfoEnabled()) logger.logInfo(leadingName + message + ": " + key) ;

 		// Calling the super method
 		manageService(key) ;
 		return true ;
 	}

	/**
	* Prints in the LOG file some information about the service configuration.<br/>
	* Notice the all classes extending the <code>GeneralService</code> class MUST implement this method.
	* Hence, in order to save coding this method forwards action to the <code>showConfiguration()</code> method
	* of the <code>ServiceMacro</code> class.
	*
	* @see ServiceMacro#showConfiguration()
	*/
	protected void showConfiguration() 			{		serviceMacro.showConfiguration() ;						}

} // END OF CLASS