/**
* Title: 		SoapHandler
* Description:	General purpose class for handling the SOAP elements handled by Axis2
* Copyright:    Copyright (c) 2010-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.util ;

import adriani.jws.template.Settings ;
import adriani.jws.template.wrappers.WrapperOut;
import adriani.jws.template.wsdl.* ;
import adriani.jws.template.service.ServiceLogic ;
import adriani.jws.template.service.ErrorMessage ;

import java.util.List ;
import java.util.regex.Pattern ;
import javax.xml.namespace.QName ;
import javax.xml.stream.XMLStreamReader ;
import javax.xml.stream.XMLStreamConstants ;

import org.apache.axiom.soap.* ;
import org.apache.axiom.om.OMElement ;
import org.apache.axis2.AxisFault ;
import org.apache.axis2.context.MessageContext ;
import org.apache.axis2.databinding.ADBException ;
import org.apache.axis2.description.AxisOperation ;
import org.apache.axis2.util.JavaUtils ;

import static javax.xml.stream.XMLStreamConstants.* ;

/**
* This class is used by the <code>XXXMessageReceiverInOut</code> handler to acknowledge data stored in
* the SOAP's header. Access to this class should be inserted in the <code>XXXMessageReceiverInOut</code>
* source file, right after the<br/><br/>
*
* <code>if("my_method".equals(methodName)) {</code><br/><br/>
*
* statement. The suggested usage is:<br/><br/>
*
* <pre><code>
* adriani.jws.template.util.SoapElements    elements        = adriani.jws.template.util.SoapHandler.getElements(msgContext) ;
* org.apache.axiom.om.OMElement             first_element   = elements.getFirstElement() ;
* String                                    header_value    = elements.getHeaderData() ;
* skel.setHeaderField(header_value) ;
* </code></pre><br/><br/>
*
* The handler class allows also to manage the SOAP response when validation
* of the input SOAP message fails. This class offers two main methods allowing to override the Axis2 behavior:
* the <code>manage()</code> method and the <code>getParsingError()</code> method.<br/><br/>
*
* The <strong><code>getParsingError()</code></strong> method should be invoked within a skeleton
* class, as produced by the wsdl2java tool. The method should be called in the <code>catch()</code>
* section at the end of the <code>parse()</code> method, right after the<br/><br/>
*
* <code>catch (javax.xml.stream.XMLStreamException e)</code><br/><br/>
*
* statement, using a code block like this:<br/><br/>
*
* <pre><code>
* catch (java.lang.Exception e) {
*    // Keep for testing or debug
*    //it.imaginary.jws.idmc.util.ParsingHandler.analyzeThrowable(e) ;
*    String msg = it.imaginary.jws.idmc.util.ParsingHandler.getParsingError(MY_QNAME, reader, e) ;
*    throw new java.lang.Exception(msg, e) ;
* }
* </code></pre><br/>
*
* The <strong><code>manage()</code></strong> method should be invoked only by the class
* <code>BtwsMessageReceiverInOut</code>. The method should be called in the <code>catch()</code>
* section at the end of the <code>invokeBusinessLogic()</code> method, by substituting the original
* exception management, which should be something like<br/><br/>
*
* <code>throw org.apache.axis2.AxisFault.makeFault(e)</code><br/><br/>
*
* The modified code block should result something like the following:<br/><br/>
*
* <pre><code>
* catch (java.lang.Exception e) {
*   org.apache.axiom.soap.SOAPFactory	factory 	= getSOAPFactory(msgContext) ;
*   org.apache.axiom.soap.SOAPEnvelope 	envelope	= adriani.jws.template.util.ParsingHandler.manage(msgContext, factory) ;
*   newMsgContext.setEnvelope(envelope) ;
* }
* </code></pre><br/><br/>
*
*/
public class SoapHandler {

	/** Static final <code>Pattern</code> object stored as class field to improve performances.				*/
	public static final 		Pattern 		pattern 		= Pattern.compile("\\.") ;

	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			RET				= "\n" ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			TAB				= "\t" ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			COLON			= ": " ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			EMPTY			= "" ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			BLANK			= " " ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			UNKNOWN			= "unknown" ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			MANAGE			= "getting SOAP envelope for method: " ;
	/** Static final <code>String</code> value stored as class fields to improve performances.				*/
	public static final			String			GET_INFORMATION		= "getInformation" ;

	/** Retrieving the logger instance to be used for this class.											*/
	protected static 			WSLogger		logger 			= WSLogger.getLogger(SoapHandler.class) ;

	/** Beginning of each logging message produced by this class.											*/
	protected static final		String 			leadingName 	= "SoapHandler - " ;

// ************************************************************************************************************
// **********************************			REQUEST MANAGEMENT		***************************************
// ************************************************************************************************************

	/**
	* Retrieves the <code>OMElement</code> representing the SOAP's body and the SOAP's header,
	* fetching them from the <code>MessageContext</code> Axis2's handler. This MUST BE done
	* before invoking others "Axis2 stuff" because the <code>MessageContext</code> has a very
	* short live and cannot be parsed twice.<br/>
	* Body and header should be (also) instances of following interfaces:<br/><br/>
	*
	* header : <code>interface org.apache.axiom.soap.SOAPHeader</code>
	* body   : <code>interface org.apache.axiom.soap.SOAPBody</code><br/><br/>
	*
	* @param msgContext the input message context used to invoke the business logic
	* @param factory a SOAP factory retrieved using <code>getSOAPFactory(msgContext)</code>
	* @throws AxisFault if an <code>ADBException</code> occurs while producing the SOAP envelope
	* @return the <code>SoapElements</code> wrapper containing all envelope's elements
	*/
	public static synchronized SoapElements getElements(MessageContext msgContext) {

    	org.apache.axiom.soap.SOAPBody 		the_body 		= msgContext.getEnvelope().getBody() ;
    	org.apache.axiom.soap.SOAPHeader 	the_header		= msgContext.getEnvelope().getHeader() ;
    	SoapElements 						elements 		= new SoapElements(the_header, the_body) ;
    	// Keep for testing or debug
		//System.out.println(leadingName + "HEADER:\n" 	+ the_header) ;
		//System.out.println(leadingName + "BODY:\n" 	+ the_body) ;
    	return elements ;
	}

// ************************************************************************************************************
// **********************************			RESPONSE MANAGEMENT		***************************************
// ************************************************************************************************************

	/**
	* Identifies the current SOAP operation (the SOAP method) in order to choose the proper output
	* wrapper to be used to produce the SOAP envelope. By this mechanism we return a complete
	* <code>SOAPEnvelope</code> object that can be set on the <code>newMsgContext</code> instance.
	* For the time being this method relies only of one wrapper: the <code>WrapperSearchOut</code>.<br/>
	* This wrapper is used to invoke the <code>wrapper.getElement()</code> method, in order
	* to get the proper <code>ADBBean</code> instance to be added the the SOAP envelope's body.
	*
	* @param msgContext the input message context used to invoke the business logic
	* @param factory a SOAP factory retrieved using <code>getSOAPFactory(msgContext)</code>
	* @throws AxisFault if an <code>ADBException</code> occurs while producing the SOAP envelope
	* @return the SOAP envelope that can be set on the <code>newMsgContext</code> object
	*/
	public static synchronized SOAPEnvelope manage(MessageContext msgContext, SOAPFactory factory) throws AxisFault {

		// 1. Getting Axis2 objects
		SOAPEnvelope 	envelope	= null ;
		AxisOperation 	operation	= msgContext.getOperationContext().getAxisOperation() ;
		String 			method 		= JavaUtils.xmlNameToJava( operation.getName().getLocalPart() ) ;

		// 2. Logging information
		StringBuffer information = new StringBuffer() ;
		information.append(	MANAGE	) ;
		information.append(	method	) ;
		logger.logInfo( information.toString() )  ;

		// 3. Getting the empty wrapper for this method
		WrapperOut			wrapper = null ;
		ServiceLogic 		logic 	= new ServiceLogic(false) ;
		if ( method.equalsIgnoreCase(GET_INFORMATION) ) 	wrapper = logic.getWrapperOut(ServiceLogic.GET_INFORMATION) ;
		// PLACE HOLDER
		//else 											wrapper = logic.getWrapperOut(ServiceLogic.NOTIFY) ;

		// 4. Packing a valid empty wrapper
		try 				{ wrapper.validate() ;						 								}
		catch(Exception e)	{ logger.logWarning(leadingName + "Failed validation of empty response") ;	}
		wrapper.forceStatus(WrapperOut.STATUS_WRONG) ;
		//wrapper.forceMessage(ErrorMessage.WRONG_XML_PARAM_MESSAGE) ;

		// 5. Building the actual SOAP envelope
		try {
			OMElement child = null ;
			// 1st method: GET_INFORMATION
			if ( method.equalsIgnoreCase(GET_INFORMATION) ) {
				Information the_output = (Information) wrapper.getElement() ;
				child = the_output.getOMElement( Information.MY_QNAME, factory) ;
			} else {
				// PLACE HOLDER (for other methods)
				//NotificheOut notificheOut = (NotificheOut) wrapper.getElement() ;
				//child = notificheOut.getOMElement( NotificheOut.MY_QNAME, factory) ;
			}
			envelope = factory.getDefaultEnvelope() ;
			envelope.getBody().addChild( child ) ;
			return envelope ;
		} catch(ADBException e) {
			throw AxisFault.makeFault(e) ;
		}
	}

	/**
	* Get detailed information about the XML validation failure from the Axis2 stream reader, in this case
	* a <code>XMLStreamReader</code> object, then uses these information to log a proper ERROR message in the
	* log system and to produce the string message describing the error. The string message could be used
	* to set the SOAP response: this is not necessary at the moment, since the error management
	* is completely performed by the <code>manage()</code> method.<br/><br/>
	*
	* Remark: if the <code>verbose</code> flag is <code>true</code> some detailed information about the
	* <code>XMLStreamReader</cif (Settings.VERBOSE) {ode> object are also logged using the INFO logger label.
	*
	* @param qName the qualified name of the input <code>ADBBean</code> who generates the error
	* @param reader the <code>XMLStreamReader</code> used to parse the SOAP request
	* @param cause an instance of a general <code>Throwable</code> object
	* @return a string describing the error, usually taken from the <code>ErrorMessage</code> class
	*/
	public static synchronized String getParsingError(QName qName, XMLStreamReader reader, Throwable cause) {

		// Initializing
		String 				output		= UNKNOWN ;
		String 				dataType	= UNKNOWN ;
		int 				type 		= reader.getEventType() ;
		StackTraceElement 	element		= getFirstElement(cause) ;

		// Getting the XSD datatype that produced the parsing error 5
		if (element != null) {
			String[] tokens	= pattern.split( element.getClassName() ) ;
			if (tokens.length > 0) dataType = new String( tokens[tokens.length-1] ) ;
		}

		// Logging mandatory information
		StringBuffer information = new StringBuffer() ;
		// TEMP: soluzione provvisoria per IDMC
		information.append("WRONG_XML_PARAM_MESSAGE") ;
		// TODO: aggiornare la classe ErrorMessage se questo metodo servisse davvero
		//information.append(	ErrorMessage.WRONG_XML_PARAM_MESSAGE	) ;
		//information.append(	BLANK									) ;
		information.append(	getSOAPMethod(qName)					) ;
		information.append(	BLANK									) ;
		information.append(	reader.getLocalName()					) ;
		information.append(	BLANK									) ;
		information.append( dataType								) ;
		logger.logSevere( information.toString() )  ;

		// Eventually logging detailed information about the reader error
		if (Settings.VERBOSE) {
			StringBuffer message = new StringBuffer() ;
			message.append(	leadingName			+ "Analyzed XMLStreamReader status:"		+ RET	) ;
			message.append(	"Has name   : " 	+ reader.hasName()							+ RET 	) ;
			message.append(	"Name       : " 	+ reader.getName()							+ RET 	) ;
			message.append(	"Local name : " 	+ reader.getLocalName()						+ RET 	) ;
			message.append(	"Event type : " 	+ getEventType(type) + " (" + type + ")"	+ RET 	) ;
			message.append(	"Caused by  : " 												+ RET 	) ;
			message.append(	getFirstElementInfo(cause)										+ RET 	) ;
			logger.logInfo( message.toString() ) ;
		}

		// Generating the string message to be set in the SOAP response
		// TODO: aggiornare la classe ErrorMessage se questo metodo servisse davvero
		//if (type == START_ELEMENT) output = ErrorMessage.WRONG_XML_PARAM_MESSAGE ;
		return output ;
	}

// ************************************************************************************************************
// **********************************			UTILITIES				***************************************
// ************************************************************************************************************

	/**
	* Return the local part of the input <code>ADBBean</code> object that generates the parsing error.
	*
	* @param qName the qualified name of an <code>ADBBean</code> instance
	* @return the name of the <code>ADBBean</code> instance
	*/
	public static String getSOAPMethod(QName qName) {

		String name = qName.getLocalPart() ;
		if (name != null) return name.substring(0, name.length() - 2) ;
		else return UNKNOWN ;
	}

	/**
	* Recognize the integer code describing the XML element pointed by a <code>XMLStreamReader</code>
	* instance and return a string message describing the XML element (or event) type. The mapping
	* between integers and string messages is retrieved from the constants defined in the
	* <code>javax.xml.stream.XMLStreamConstants</code> class.
	*
	* @param id the integer number retrieved from the <code>reader.getEventType()</code> method
	* @return the string message describing the event: for example "1" should be "START_ELEMENT"
	*/
	public static String getEventType(int id) {

		switch(id) {
			case START_ELEMENT	: 	return "START_ELEMENT" ;
			case END_ELEMENT	: 	return "END_ELEMENT" ;
			case ATTRIBUTE		: 	return "ATTRIBUTE" ;
			default 			: 	return (EMPTY + id) ;
		}
	}

	/**
	* Return the first <code>StackTraceElement</code> instance from the stack trace element bound to
	* the input <code>Throwable</code> object.
	*
	* @param cause an instance of a general <code>Throwable</code> object
	* @return a valid <code>StackTraceElement</code> instance
	*/
	public static StackTraceElement getFirstElement(Throwable cause) {

		StackTraceElement 			element 	= null ;
		StackTraceElement[] 		stackTrace 	= cause.getStackTrace() ;
		if (stackTrace.length > 0) 	element 	= stackTrace[0] ;
		return element ;
	}

	/**
	* Return some information (already formatted) about the first <code>StackTraceElement</code> of the
	* the input <code>Throwable</code> stack array. These information are retrieved by invoking the
	* <code>getElementInfo()</code> method.
	*
	* @param cause an instance of a general <code>Throwable</code> object
	* @return some information (already formatted) about the first stack trace element
	*/
	public static StringBuffer getFirstElementInfo(Throwable cause) {

		return getElementInfo( getFirstElement(cause) ) ;
	}

	/**
	* Return some information (already formatted) about the input <code>StackTraceElement</code> instance.
	*
	* @param element a valid <code>StackTraceElement</code> instance
	* @return some information (already formatted) about the stack trace element
	*/
	public static StringBuffer getElementInfo(StackTraceElement element) {

		StringBuffer message = new StringBuffer() ;
		if (element != null) {
			message.append(TAB	+	"Class  : " 	+ element.getClassName()	+ 	RET	) ;
			message.append(TAB	+	"File   : " 	+ element.getFileName()		+ 	RET	) ;
			message.append(TAB	+	"Line   : " 	+ element.getLineNumber()	+ 	RET	) ;
			message.append(TAB	+	"Method : " 	+ element.getMethodName()	+ 	RET	) ;
		} else {
			message.append("Found null StackTraceElement" + RET) ;
		}
		return message ;
	}

// ************************************************************************************************************
// **********************************			TEST SECTIONS			***************************************
// ************************************************************************************************************

	/**
	* Produce some logging information about a general <code>Throwable</code> object.<br/>
	* This method can be used at the end of the method <code>fromOM()</code> of the message receiver class
	* (in this case a <code>BtwsMessageReceiverInOut</code> instance) in order to obtain information about the
	* general <code>Exception</code> used to invoke the <code>org.apache.axis2.AxisFault.makeFault(e)</code>
	* method. Example:<br/><br/>
	*
	* <pre><code>
	* catch (java.lang.Exception e) {
	* 	//adriani.jws.template.util.ParsingHandler.analyzeThrowable(e) ;	// Keep for testing or debug
	*	throw org.apache.axis2.AxisFault.makeFault(e) ;						// Original code
	*	//throw adriani.jws.template.util.ParsingException.makeFault(e) ;	// Modified code
	* }
	* </code></pre>
	*
	* @param cause an instance of a general <code>Throwable</code> object
	* @return the formatted string message containing the analysis results
	* @see BtwsMessageReceiverInOut#fromOM(org.apache.axiom.om.OMElement param, java.lang.Class type, java.util.Map extraNamespaces)
	*/
	public static synchronized StringBuffer analyzeThrowable(Throwable cause) {

		StringBuffer				message		= null ;
		StackTraceElement 			element 	= null ;
		StackTraceElement[] 		stackTrace 	= cause.getStackTrace() ;
		if (stackTrace.length > 0) 	element 	= stackTrace[0] ;

		// Logging general information about the Throwable object
		message = new StringBuffer() ;
		message.append(leadingName			+ "Analyzed Throwable object:" 	+ 	RET	) ;
		message.append("Instanceof    : "	+ cause.getClass()				+	RET	) ;
		message.append("Cause         : " 	+ cause.getCause()				+	RET	) ;
		message.append("Localized     : "	+ cause.getLocalizedMessage()	+	RET	) ;
		message.append("Message       : "	+ cause.getMessage()			+	RET	) ;
		message.append("Stack length  : "	+ stackTrace.length				+	RET	) ;
		message.append("First element : "	+ element						+	RET	) ;
		//for (StackTraceElement ste : stackTrace) message.append("\t" + ste + RET) ;

		// Adding information about the first stack trace element
		message.append( getElementInfo(element) ) ;

		// Analyzing an AxisFault (this can never happen at the moment, see below)
		if (cause instanceof AxisFault) analyzeAxisFault(cause) ;

		// Exposing analysis results
		logger.logInfo( message.toString() ) ;
		return message ;
	}

	/**
	* NOT USED AT THE MOMENT!<br/><br/>
	*
	* This method it's never invoked because the classes generated by the "wsdl to java" Axis2 tool
	* do not create any <code>AxisFault</code> exception when parsing the SOAP request, neither
	* when invoking the skeleton business logic. The <code>AxisFault</code> faults are created only
	* by the <code>BtwsMessageReceiverInOut</code> handler class, after "the point" where we want to
	* insert the management of general parsing errors (and the "too late").
	*
	* @param cause an instance of a general <code>Throwable</code> object
	*/
	public static synchronized void analyzeAxisFault(Throwable cause) {

		AxisFault 		fault 		= 	(AxisFault) cause ;
		OMElement 		element 	= 	fault.getDetail() ;
		String 			action 		=	fault.getFaultAction() ;
		QName			qName		=	fault.getFaultCode() ;
		SOAPFaultCode	soapCode	= 	fault.getFaultCodeElement() ;
		SOAPFaultDetail	soapDetail	=	fault.getFaultDetailElement() ;
		List			list		= 	fault.getFaultSubCodes() ;

		// Next lines have not been implemented to avoid to import all the required packages,
		// since this method is not used at the moment (see method's header)

		/*
		fault.getFaultReasonElement() ;
		fault.getFaultRoleElement() ;
		fault.getNodeURI() ;
		fault.getFaultMessageContext() ;
		fault.getFaultNode() ;
		fault.getFaultNodeElement() ;
		fault.getFaultRole() ;
		fault.getFaultType() ;
		fault.getMessage() ;
		fault.getReason() ;*/
	}

} // End of class