/**
* Title: 		Utilities
* Description:	General purposes class for conversion of time, date or string formats
* Copyright:    Copyright (c) 2010-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.util ;

import java.util.* ;
import java.text.SimpleDateFormat ;

/**
* General utilities class for conversion of different formats, typically date and time formats.<br/>
* Let's recall the standard Java date and time lexicon:<br/><br/>
*
* <strong>Calendar.ZONE_OFFSET</strong>: this offset represent the offset of the date specified
* in the data string description, for example "+01:00" or "+02:00".<br/>
* Example: the date "2010-02-26T15:40:00+00:00" will be converted into a calendar with offset equals 0,
* the date "2010-02-26T15:40:00+01:00" will be have an offset set to 3600000, while the date
* "2010-02-26T15:40:00+02:00" will be have an offset set to 7200000.<br/><br/>
*
* <strong>TimeZone</strong>: describes the time zone bound to a <code>Calendar</code> object.<br/>
* If the user changes the machine setting (e.g. on Windows: setting daylight saving in the control panel)
* the default <code>TimeZone</code> object will immediately reflect this change, as visible for example
* by calling the <code>useDaylightTime()</code> method. Instead, the raw offset of the time zone will
* not be affected by such a change, since it describes the difference with respect of the UTC time.
* In other words, the raw offset for the Italian time zone is always 3600000, and will never change.<br/>
* Notice that usually a calendar relies on the local <strong>default</strong> time zone instead of using
* the custom time zone depicting where the calendar was created (fortunately).<br/><br/>
*
*/
public abstract class Utilities {

	/** The data pattern used by Log4j for the timestamp, in this case: yyyy-MM-dd HH:mm:ss,SSS				*/
	protected static 		SimpleDateFormat 	sdf_log 			= new SimpleDateFormat("yyyy-MM-dd HH:mm:ss,SSS") ;
	/** The data pattern used by Log4j for the timestamp, in this case: yyyy-MM-dd HH:mm:ss,SSS				*/
	protected static 		SimpleDateFormat 	sdf_ita 			= new SimpleDateFormat("dd-MMM-yy HH:mm:ss,SSS", Locale.ITALY) ;
	/** The data pattern used as output for the time, in this case: HH:mm:ss,SSS							*/
	protected static 		SimpleDateFormat 	sdf_time 			= new SimpleDateFormat("HH:mm:ss") ;
	/** The data pattern used as output for the date, in this case: yyyy-MM-dd HH							*/
	protected static 		SimpleDateFormat 	sdf_date 			= new SimpleDateFormat("dd-MMM-yy") ;

	/**
	* SimpleDateFormat static reference for conversion from the format yyyy-MM-dd'T'HH:mm:ss'+00:00',
	* referred to the standard UTC time.
	*/
	protected static 		SimpleDateFormat 	sdf_utc				= new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'+00:00'") ;
	/**
	* SimpleDateFormat static reference for conversion from the format yyyy-MM-dd'T'HH:mm:ss'+01:00',
	* used in Italy when NOT in the saving light season.
	*/
	protected static 		SimpleDateFormat 	sdf_winter 			= new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'+01:00'") ;
	/**
	* SimpleDateFormat static reference for conversion from the format yyyy-MM-dd'T'HH:mm:ss'+02:00',
	* used in Italy DURING the saving light season.
	*/
	protected static 		SimpleDateFormat 	sdf_summer 			= new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'+02:00'") ;


  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 				leadingName 		= "Utilities - " ;

	/**
	* Due the Axis2 design the <code>Calendar</code> obtained from a SOAP REQUEST does not consider the
	* time zone, but it's created using the local time zone, which for Italy is GMT+01:00
	* (Central Europe Time) in winter and GMT+02:00 (Central Europe Summer Time) in summer.
	* For this reason, the time zone of the input datetime parameter will be specified in the
	* <code>Calendar.ZONE_OFFSET</code> field.<br>
	* Example: if the SOAP input date is like "2010-02-26T15:40:00+02:00", then the calendar
	* object will have an offset of 7200000 ms.<br/><br/>
	*
	* This method allows to emulate the Axis2 behavior when taking datetime values from another source,
	* like an hard-coded string array or a properties file.<br/>
	*
	* @param value a <code>string</code> object describing a <code>Date</code>
	* @return the same date converted into an equivalent <code>Calendar</code> object
	*/
	public static Calendar getCalendar(String value) throws java.text.ParseException {

		Date 		time		= null ;
		char 		type		= value.charAt(21) ;
		Calendar	calendar	= Calendar.getInstance() ;
		System.out.println(leadingName + "Parsing date: " + value + " [" + type + "]") ;

		// Emulating behavior of the SOAP input date, which set the time zone offset in the calendar
		// and does not set the daylight saving offset.
		switch(type) {
			case	'1'	:	time = sdf_winter.parse(value) ;
							calendar.setTime(time) ;
							calendar.set(Calendar.DST_OFFSET, 0) ;
							calendar.set(Calendar.ZONE_OFFSET, 3600000) ;
							break ;
			case	'2'	:	time = sdf_summer.parse(value) ;
							calendar.setTime(time) ;
							calendar.set(Calendar.DST_OFFSET, 0) ;
							calendar.set(Calendar.ZONE_OFFSET, 7200000) ;
							break ;
			default		:	time = sdf_utc.parse(value) ;
							calendar.setTime(time) ;
							calendar.set(Calendar.DST_OFFSET, 0) ;
							calendar.set(Calendar.ZONE_OFFSET, 0) ;
		}

		// This date will be the straight conversion, without offset, always in CET format (defaul)
		//System.out.println(leadingName + "Parsed date: " + time) ;
		// This calendar will use the local machine time zone
		//System.out.println(leadingName + "Inspecting calendar: " + Utilities.introspectElement(calendar)) ;

		return calendar ;
	}

 // *********************************************************************************************************
 // **********************************			DATE FORMATS			*************************************
 // *********************************************************************************************************

    /**
	* Convert the input <code>Calendar</code> to the string format, using the format
	* <code>dd-MMM-yy</code>.
	*
	* @input calendar the calendar to be converted
	* @return the formatted version of the input calendar
	* @see sdf_date
    */
    public static String getShortDate(Calendar calendar) {

    	if (calendar == null)	return null ;
    	else					return getShortDate(calendar.getTimeInMillis()) ;
    }

    /**
	* Convert the input <code>timestamp</code> to the string format, using the format
	* <code>dd-MMM-yy</code>.
	*
	* @input timestamp the timestamp to be converted
	* @return the formatted version of the input timestamp
	* @see sdf_date
    */
    public static String getShortDate(long timestamp) {

		try {
			Date date = new Date(timestamp) ;
			return sdf_date.format(date) ;
		} catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}

 // *********************************************************************************************************
 // **********************************			TEST OR DEBUG			*************************************
 // *********************************************************************************************************

	/**
	* Utility for printing introspection information about the ADBBeans elements.
	* For the time being this method support introspection only for datetime objects, in order to
	* investigate behavior of an Axis2 strange behavior:<br/><br/>
	*
	* https://issues.apache.org/jira/browse/AXIS-2673?page=com.atlassian.jira.plugin.system.issuetabpanels:all-tabpanel
	* <br/><br/>
	*
	* @param object the XML node value retrieved from a <code>ADBBean</code> object
	* @return a formatted string ready to be printed on the output stream
	*/
	public static String introspectElement(Object object) {

		Date			today	= new Date() ;
		String			RET		= new String("\n") ;
		StringBuffer 	text 	= new StringBuffer(RET) ;

		// Inspecting calendar objects
		if (object instanceof Calendar) {
			Calendar calendar 	= (Calendar) object ;
			Date 	date		= calendar.getTime() ;
			TimeZone timeZone 	= calendar.getTimeZone() ;
			//text.append(calendar.toString() + RET) ;
			text.append(	"TIMEZONE        : "							) ;
			text.append(	timeZone.toString()						+	RET	) ;
			text.append(	"CALENDAR DAY    : "							) ;
			text.append(	calendar.get(Calendar.DAY_OF_MONTH) + "-" + calendar.get(Calendar.MONTH) 	+ RET ) ;
			text.append(	"CALENDAR HOUR   : "							) ;
			text.append(	calendar.get(Calendar.HOUR_OF_DAY) 	+ ":" + calendar.get(Calendar.MINUTE)	+ RET ) ;
			text.append(	"CALENDAR OFFSET : "							) ;
			text.append(	calendar.get(Calendar.ZONE_OFFSET)		+	RET	) ;
			text.append(	"CALENDAR DST    : "							) ;
			text.append(	calendar.get(Calendar.DST_OFFSET)		+	RET	) ;
			text.append(	"TIME ZONE ID    : "							) ;
			text.append(	timeZone.getID()						+	RET	) ;
			text.append(	"NAME            : "							) ;
			text.append(	timeZone.getDisplayName()				+	RET	) ;
			text.append(	"USE DAYLIGHT    : "							) ;
			text.append(	timeZone.useDaylightTime()				+	RET	) ;
			text.append(	"IN DAYLIGHT     : "							) ;
			text.append(	timeZone.inDaylightTime(date)			+	RET	) ;
			text.append(	"RAW OFFSET      : "							) ;
			text.append(	timeZone.getRawOffset()					+	RET	) ;
			text.append(	"SYSTEM DATE     : "							) ;
			text.append(	today									+	RET	) ;
			text.append(	"CALENDAR DATE   : "							) ;
			text.append(	date									+	RET	) ;
		}
		else return "Unsopported" ;

		return text.toString() ;
	}

} // End of class