/**
* Title: 		Wrapper
* Description:	General wrapper for all the <code>ADBBean</code> objects created by the wsdl2java tool
* Copyright:    Copyright (c) 2010-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.wrappers ;

import java.math.BigInteger;

import adriani.jws.template.util.WSLogger ;

/**
* Wrapper - general class wrapping all the <code>ADBBean</code> elements describing the "inner core" of
* the SOAP response, i.e. a set of fields bundled together to create a record-like structure.
* Usually this set is wrapped inside another <code>WrapperIn</code> or <code>WrapperOut</code> object,
* for example :<br/><br/>
*
* <pre><code>
* &lt;WrapperOut&gt;
*   status  (1 field)
*   message (1 field)
*   records (1..n occurrences of the &lt;Wrapper&gt; object)
* &lt;/WrapperOut&gt;
* </code></pre>
*/
public abstract class Wrapper {

 	/** The wrapped <code>ADBBean</code> element.															*/
	protected 				Object 				element 			= null ;

  	/**
	* Static <strong>string</strong> value depicting a undefined number field.<br/>
	* Example: if validation fails for some input numeric field, and if the corresponding class attribute
	* (of this wrapper) is a string type, the <code>NO_NUMBER</code> string is stored into the class attribute
	* in order to acknowledge this situation.<br/>
	* Current implementation stores the text <code>UNSET</code> in the NO_NUMBER field.
	*/
	public static 		 	StringBuffer 		NO_NUMBER 			= new StringBuffer("UNSET") ;
  	/** Numeric reference value for all the empty (null) integer values.									*/
	public static 		 	int 				NOT_AVAILABLE		= 0 ;
  	/** Static <strong>BigInteger</strong> value depicting a undefined number field.						*/
	public static final 	BigInteger			NOT_BIG_INTEGER		= BigInteger.valueOf(-1) ;
  	/** String reference value for all the empty (null) text values.										*/
	public static 		 	StringBuffer		UNDEFINED			= new StringBuffer("UNDEFINED") ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 				leadingName 		= "Wrapper - " ;

	/** Retrieving the logger instance to be used for this class (static).									*/
	protected static 		WSLogger			logger 				= WSLogger.getLogger(Wrapper.class) ;

// **********************************************************************************************************
// **********************************		ADBBEAN MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Validation is performed by the Axis2 <code>ADBBean</code> objects ( e.g. NameType.setNameType() ).
	* If the validation process is successful, the resulting <code>ADBBean</code> object is stored as
	* class field to save computation time, since after validation we expect that the method
	* <code>getElement()</code> should be invoked.<br/><br/>
	*
	* <strong>Remark:</strong> to grant LOOSE COUPLING to the WSDL, the <code>ADBBean</code> Java classes
	* should be used only within this method, not elsewhere.
	*
	* @return <code>true</code> if all the mandatory <code>ADBBean</code> objects have been set correctly
	*/
	public abstract boolean validate() throws WrapperFault ;

	/**
	* Load all the STATIC default values (defined in this class) into the corresponding class attributes.
	* This method should be called by the constructor if we need to make sure that we "empty wrapper" contains
	* always some good default values matching the WSDL constrains.
	*/
	public abstract void setDefaults() ;

	/**
	* Return the <code>ADBBean</code> object wrapped by this class. Usually this method is called
	* after validation, if successful.
	*
	* @return a reference to the <code>ADBBean</code> object wrapped by this class
	*/
	public Object getElement() 					{		return this.element ;								}

// **********************************************************************************************************
// **********************************			UTILILTIES			*****************************************
// **********************************************************************************************************

	/**
	* Helper method, used to compare the value of two <code>StringBuffer</code> objects.
	* Usually this method is used by <code>isXXX()</code> methods or <code>hasXXX()</code> methods
	* of the children classes.
	*
	* @param first a <code>StringBuffer</code> object
	* @param second another <code>StringBuffer</code> object
	* @return <code>true</code> if the two string object contains the same text
	*/
	public boolean compareSB(StringBuffer first, StringBuffer second) {

		String 	string 	= null ;
		boolean result	= false ;
		if (first != null) 	string = first.toString() ;
		if (string != null) result = string.equalsIgnoreCase( second.toString() ) ;
		return result ;
	}

} // End of class