/**
* Title: 		WrapperIn
* Description:	Class wrapping the input request for the getInformation method
* Copyright:    Copyright (c) 2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.wrappers ;

import adriani.jws.template.Settings;
import adriani.jws.template.wsdl.*;

import java.math.* ;

/**
* The class wrapping the <code>ADBBean</code> elements containing the input SOAP request for the
* <code>getInformation()</code> method.
*/
public class WrapperInformationIn extends WrapperIn {

   	/** Wrapped input value, used to store the SOAP input request value after validation.						*/
	protected 				StringBuffer	number ;
	/** Wrapped input value, used to store the SOAP input request value after validation.						*/
	protected 				BigInteger		customer ;

	/**
	* Constructor used by the <code>SimpleClient</code> to test the service (empty).
	*/
	public WrapperInformationIn() {

		WrapperInformationIn.leadingName = "WrapperInformationIn - " ;
	}

	/**
	* Constructor used by the actual web service logic, used to set the wrapper's class fields reading
	* the values from the <code>ADBBean</code> element of the input SOAP request. For this reason
	* implementation of this constructor is strongly coupled to the WSDL definition.
	*
	* @param element the <code>ADBBean</code> object as received by the service skeleton class
	*/
	public WrapperInformationIn(User user) {

		// Setting general purpose class fields
		this() ;

		// Retrieving ADBBean root element
		UserType userType 	= user.getUser() ;

		// Retrieving ADBBean inner types
		PhoneType phoneType = userType.getPhone() ;

		// Adjusting input value (optional)
		BigInteger 		customer 					= userType.getCustomer() ;
		String 			phone						= phoneType.getPhoneType().trim() ;

		// Setting class fields (without validation in order to acknowledge ADB values)
		if (customer 	!= null) 	this.customer	= customer ;
		if (phone	 	!= null) 	this.number		= new StringBuffer( phone ) ;
		//System.out.println(leadingName + "Set customer ID: " + this.customer) ;
	}

// **********************************************************************************************************
// **********************************		ELEMENT MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* This method should be called after that all the wrapper's fields have been properly set, in order to
	* adjust any string requiring a particular format. For example, if the datasource required that all the
	* accent vowels must be replaced with the corresponding single quote notation, this method should take
	* care to implement the necessary operations (converting <em></em> into <em>a'e'e'i'o'u'</em>).<br/><br/>
	*
	* Current implementation does nothing.<br/><br/>
	*
	* <strong>Remark:</strong> to grant LOOSE COUPLING to the WSDL, the <code>ADBBean</code> Java classes
	* should NOT be used within this method.
	*
	* @return <code>true</code> if all the mandatory <code>ADBBean</code> objects were set correctly
	*/
	public boolean validate() {

		// Example code
		//if (customer < 0) customer = - customer ;
		return true ;
	}

	/**
	* Doing nothing (implementation required by the parent interface definition).<br/>
	* Usually SOAP methods does not need to set default data in the input SOAP requests, since
	* if some data is missing we can not assume some default behavior.<br/><br/>
	*
	* See children of the <code>WrapperOut</code> class for some actual usage of this method.
	*
	* @see Wrapper()#Wrapper
	* @see WrapperOut
	*/
	public void setDefaults() {

		if (Settings.VERBOSE) System.out.println(leadingName + "setDefaults: doing nothing") ;
	}

// **********************************************************************************************************
// **********************************			ACCESSORS		*********************************************
// **********************************************************************************************************

	/**
	* Accessor method.
	*/
	public BigInteger getCustomer()						{		return this.customer ;						}

	/**
	* Accessor method.
	*/
	public StringBuffer getNumber()						{		return this.number ;						}

 // **********************************************************************************************************
// **********************************			MUTATORS		*********************************************
// **********************************************************************************************************

	/**
	* Mutator method.
	*/
	public void setCustomer(BigInteger value)			{		this.customer = value ;						}

	/**
	* Mutator method (overloaded).
	*/
	public void setCustomer(StringBuffer value) {

		try 				{ this.customer = new BigInteger( value.toString() ) ;							}
		catch(Exception e) 	{ this.customer = NOT_AVAILABLE ;												}
		//try 				{ this.customer = Integer.parseInt( value.toString() ) ;						}
		//catch(Exception e) 	{ this.customer = NOT_AVAILABLE ;												}
		//System.out.println(leadingName + "Set customer ID: " + this.customer) ;
	}

	/**
	* Mutator method.
	*/
	public void setNumber(StringBuffer value)			{		this.number = value ;						}

// **********************************************************************************************************
// **********************************			BOOLEANS			*****************************************
// **********************************************************************************************************

	/**
	* Helper method, used just to improve performance by reducing multiple checks (only for frequent calls).
	*/
	public boolean isGoodNumber() 				{	return ! compareSB(number, NO_NUMBER) ;					}

// **********************************************************************************************************
// **********************************			UTILITIES			*****************************************
// **********************************************************************************************************

	/**
	* Serialize all the data contained by this wrapper as a <code>String</code> object, dot separated.
	*
	* @return a string of the type "scott.tiger.42.prince consort road.london"
	*/
	public String toString() {

		StringBuilder 	builder = new StringBuilder(this.DOT) ;
		builder.append(this.number) ;
		builder.append(this.DOT) ;
		builder.append(this.customer) ;
		return builder.toString() ;

		// TODO: tenere per Wrapper 2 metodo
		//String		date	= null ;
		//if ( (date = Utilities.getShortDate(this.borndate)) != null) builder.append(date) ;
	}

	/**
	* Serialize all the data contained by this wrapper as a <code>String</code> object.
	* This is the "old plain version" of the <code>toString()</code> method, which can be
	* used only if ALL input data (contained the SOAP request) are mandatory!
	*
	* @return a string of the type "rome.rm.via.garibaldi.12"
	*/
	public String arrayToString() {

		StringBuffer[]	the_array	= this.toArray(false) ;
		StringBuffer 	the_string 	= new StringBuffer(DOT) ;
		if (the_array != null) for (StringBuffer sb : the_array) {
			the_string.append(sb) ;
			the_string.append(DOT) ;
		}
		return the_string.toString() ;
	}

	/**
	* Produce a <code>StringBuffer[]</code> array containing all data contained by this wrapper.
	* If this method must be used in a test environment (where dummy input data should be complete),
	* the input flag <code>complete</code> allows to add some hard-coded values if the creation
	* of the array fails.
	*
	* @param complete if <code>true</code> some hard-coded data are created if creation of the array fails
	* @return a string buffer array with the data contained by this wrapper
	*/
	public StringBuffer[] toArray(boolean complete) {

		StringBuffer[] 		the_array 	= null ;
		try {
			StringBuffer 	customer 	= new StringBuffer		( "" + this.customer 	) ;
							the_array 	= new StringBuffer[] 	{number, customer		} ;
			// TODO: tenere per Wrapper 2 metodo
			//StringBuffer born_date 	= new StringBuffer		( Utilities.getShortDate(borndate)									) ;
		}
		catch(Exception e) {
			// Setting some dummy hard-coded values
			if (complete) {
				String[] hardCoded	= new String[] 			{ "number","customer" } ;
				the_array			= new StringBuffer[hardCoded.length] ;
				for (int i = 0 ; i < hardCoded.length ; i++) the_array[i] = new StringBuffer(hardCoded[i]) ;
			}
			if (Settings.VERBOSE) System.out.println(leadingName + "Impossible to convert all input data to a valid array: some data missing?") ;
		}
		return the_array ;
	}

	/**
	* Special accessor used to retrieve the field used to send commands to the web service. This special field
	* must be a text type (String or StringBuffer) to allow inserting a "leading password" in the field, for
	* example "000". Whatever is the type of the special field, this method returns always a
	* <code>StringBuffer</code> to ensure encapsulation.
	*
	* @see ServiceLogic#manageService(WrapperSearchIn input)
	* @return a <code>StringBuffer</code> object containing the original field value
	*/
	public StringBuffer getCommand() {
			if (this.number == null) 	return null ;
			else						return new StringBuffer(this.number) ;
	}

} // End of class