/**
* Title: 		WrapperDataOut
* Description:	Simple wrapper class for the whole SOAP response
* Copyright:    Copyright (c) 2009-2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.wrappers ;

import adriani.jws.template.* ;
import adriani.jws.template.service.* ;
import adriani.jws.template.wsdl.*;

/**
* The class wrapping the root <code>ADBBean</code> output element representing the results for
* the <code>getInformation()</code> SOAP method.
* In this case the response contains also some metadata (e.g. the status message) and an array
* of records: each of these records should be wrapped by a single <code>WrapperInformation</code> object.
*/
public class WrapperInformationOut extends WrapperOut {

  	/** Placeholder: example of a custom value wrapped at this level								*/
	//protected 				int			 		value_id ;

	/**
	* Stateless constructor, loading the default values into this wrapper. This means that all the class
	* attributes are defined, but no <code>ADBBean</code> element is instanciated, because we delegate
	* this task to the <code>validate()</code> method in order to grant loose coupling.
	*
	* @see setDefaults()
	*/
	public WrapperInformationOut() {

		WrapperInformationOut.leadingName = "WrapperDataOut - " ;		// MUST BE first line
		this.setDefaults() ;
	}

	/**
	* Stateful constructor, creating an <code>Wrapper</code> set with the input array.
	*
	* @param array the array of <code>Wrapper[]</code>
	*/
	public WrapperInformationOut(Wrapper[] array) {

		this.records = array ;
	}

// **********************************************************************************************************
// **********************************		ELEMENT MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Validation is performed by the Eclipse's <code>ADBBean</code> objects (e.g. TownType.setTownType() etc.)
	* If the validation process is succesfull, the resulting <code>ADBBean</code> object is stored as
	* class field to save computation time, since after validation we expect that the method
	* <code>getElement()</code> should be invoked.<br/><br/>
	*
	* <strong>Remark:</strong> to grant LOOSE COUPLING to the WSDL, the <code>ADBBean</code> Java classes
	* should be used only within this method and in the <code>forceStatus()</code>,
	* <code>forceMessage()</code> methods, not elsewhere.
	*
	* @return <code>true</code> if all the mandatory <code>ADBBean</code> objects have been set correctly
	*/
	public boolean validate() throws WrapperFaultOut {

		// 1. Preparing variables
		String msg 		= null ;
		int max			= this.records.length ;

		// 2. Defining ADBBean types
		Information 				root 						= new Information() ;
		StatusType					statusType					= new StatusType() ;
		InformationType 			informationType				= new InformationType() ;
		RecordsType[]				array						= new RecordsType[max] ;
		//System.out.println(leadingName + "Created all ADBBean types") ;

		// 3. Setting the array sequence of all wrappers
		for (int i = 0 ; i < max ; i++) {
			// Trying to validate all the inner <code>Wrapper</code> objects
			try {
				records[i].validate() ;
			} catch(WrapperFault e) {
				if (msg == null) 	msg = 			e.getMessage() + " (" + i + ')' ;
				else 				msg += 	", " +	e.getMessage() + " (" + i + ')' ;
			}
			// Setting the single array element, which can not throw exceptions
			array[i] = (RecordsType) this.records[i].getElement() ;
		}

		// 4. If validation fails, we'll delegate ServiceLogic.handleNoValidation() for setting the message
		// Here we must avoid to overwrite any (eventual) status/message set before.
		if (msg == null) {
			if (status 	== STATUS_UNSET) 	status 	= STATUS_OK ;
			if (message == MESSAGE_UNSET) 	message = MESSAGE_OK ;
		}

		// 5. Trying to set the STATUS value into the corresponding ADBBean element
		int value = this.status ;
		for (int i = 0 ; i < 2 ; i++) {
			try 				{	statusType.setStatusType(value) ; 		break ;							}
			catch(Exception e)	{ 	value = STATUS_WRONG ;					msg = "STATUS" ;				}
		}

		// 6. Setting the main "root element" (which can not throw exceptions)
		if (status != STATUS_UNSET) informationType.setStatus( statusType ) ;
		if (message != null) 		informationType.setMessage( message.toString() ) ;
		if (array != null)			informationType.setRecords( array ) ;
		root.setInformation( informationType ) ;

		// 7. Storing root element as class attribute (see parent class)
		this.element = (Object) root ;

		// 8. Eventually forwarding a validation error to the upper level
		if (msg == null) {
			if (Settings.VERBOSE) System.out.println(leadingName + "Wrapped a valid output element") ;
		} else {
			if (Settings.VERBOSE) System.out.println(leadingName + "Invalid output wrapper, using defaults") ;
			throw new WrapperFaultOut(msg) ;
		}

		// 9. Returning validation result
		return (msg == null) ;
	}

	/**
	* Load all the STATIC default values (defined in this class) into the corresponding class attributes,
	* in this case we create one element array with just one <code>WrapperInformation</code> object
	* (set with his default values), then we set the general purposes class fields
	* (i.e. <em>status</em> and <em>message</em>).
	*/
	public void setDefaults() {

		this.status			= STATUS_UNSET ;
		this.message		= MESSAGE_UNSET ;
		this.records 		= new WrapperInformation[1] ;
		this.records[0]		= new WrapperInformation() ;
		if (Settings.VERBOSE) System.out.println(leadingName + "Set defaults values") ;
	}

	/**
	* Set an integer value in the wrapped ADBBean element, without checking for validation and ignoring the
	* value of the <code>status</code> class attribute. This method is used when validation fails and it's
	* necessary to set the response status in the status field of the wrapped ADBBean element.
	*
	* @param status the response status value
	*/
	public void forceStatus(int value) {

		// Setting the value in the inner ADBBean element
		StatusType statusType = new StatusType() ;
		try {
			statusType.setStatusType(value) ;
		} catch(Exception e) {
			e.printStackTrace() ;
			statusType.setStatusType(WrapperOut.STATUS_WRONG) ;
			System.err.println(leadingName + "Impossible to force the response status") ;
		}

		//Setting status flag in the actual ADBBean element, without validation issues
		Information root = (Information) this.element ;
		InformationType informationType = root.getInformation() ;
		informationType.setStatus( statusType ) ;
	}

	/**
	* Set a string message in the inner <code>ADBBean</code> element, without checking for validation and
	* overwriting the previous value of the <code>message</code> class attribute (in any).
	* This method is used when validation fails and it's necessary to communicate the error in a message field
	* of the wrapped <code>ADBBean</code> element.<br/><br/>
	*
	* <strong>Remark:</strong> this method MUST be called only after validation, because we want to
	* put all the building logic of the <code>ADBBean</code> structure in the <code>validate()</code> method.
	*
	* @param message the validation error message
	*/
	public void forceMessage(String message) {

		// Store message in the class field (necessary for testing from the SimpleClient)
		this.message = new StringBuffer(message) ;

		// Setting message in the actual ADBBean element, without validation issues
		Information root = (Information) this.element ;
		InformationType informationType = root.getInformation() ;
		informationType.setMessage(message)	;
	}

// **********************************************************************************************************
// **********************************			ACCESSORS			*****************************************
// **********************************************************************************************************

	/**
	* Accessor method (example).
	*/
	//public int getValueID()							{		return this.contact_id  ;					}

// **********************************************************************************************************
// **********************************			MUTATORS		*********************************************
// **********************************************************************************************************

	/**
	* Accessor method (example).
	*/
	//public void setValueID(int	value)				{		this.contact_id = value ;					}

} // End of class