/**
* Title: 		WrapperOut
* Description:	Parent class for all wrappers of whole SOAP response
* Copyright:    Copyright (c) 2012
* Company:      Morpheus Quality Technologies
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.jws.template.wrappers ;

import adriani.jws.template.Settings ;
import adriani.jws.template.service.* ;

/**
* WrapperOut - general class wrapping the root <code>ADBBean</code> element containing a general
* output result. This class should be the general parent class for all output wrappers.<br/><br/>
*
* This class can wrap an actual array of <code>Wrapper</code> objects, a single <code>Wrapper</code> object
* or none other wrapper (if the SOAP response does not contain any nested structure).
* In this way if the WSDL specifications changed the whole architecture
* should not change: to change from an array-oriented response to a simple plain response just change the
* implementation the <code>validate()</code> method.
*/
public abstract class WrapperOut extends Wrapper {

	/** The array of the "inner wrappers" stored as CLASS FIELD before validation.							*/
	protected 				Wrapper[] 			records ;

  	/** The integer code describing the status of the response.												*/
	protected 				int			 		status ;
  	/** The text message describing the status of the response.												*/
	protected 				StringBuffer 		message ;
  	/** The text message describing the status of the response.												*/
	protected 				StringBuffer 		details ;

  	/** Static reference value for the STATUS field.														*/
	public static  			int 				STATUS_OK			=  1 ;
	/** Static reference value for the STATUS field.														*/
	public static  			int					STATUS_WRONG		=  0 ;
	/** Static reference value for the STATUS field.														*/
	public static 	 		int 				STATUS_UNSET		= -1 ;

	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_OK			= new StringBuffer( ErrorMessage.getMessage(ErrorMessage.OK) ) ;
	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_NOK			= new StringBuffer( ErrorMessage.getMessage(ErrorMessage.NOK) ) ;
	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_WRONG		= new StringBuffer( ErrorMessage.getMessage(ErrorMessage.GENERAL) ) ;
	/** Static reference values for the MESSAGE field.														*/
	public static 			StringBuffer 		MESSAGE_UNSET		= null ;

	/** Static reference values for the VERBOSE field.														*/
	public static 			StringBuffer 		NO_DETAILS			= new StringBuffer("NO_DETAILS") ;

  	/** Beginning of each logging message produced by this class.											*/
	protected static 		String 				leadingName 		= "WrapperOut - " ;

	/**
	* Stateless constructor, loading the default values into this wrapper. This means the all the class
	* attributes are defined, but no <code>ADBBean</code> element is instantiated, because we delegate
	* this task to the <code>validate()</code> method, in order to grant loose coupling.
	*
	* @see setDefaults()
	*/
	public WrapperOut() {

		this.setDefaults() ;
	}

	/**
	* Stateful constructor, creating an <code>WrapperSearch</code> set with the input array.<br/>
	*
	* <strong>Remark:</strong> this method MUST NOT set the defaults values, since for this service we
	* don't want to return any message if everything goes fine
	*
	* @param array the array of <code>WrapperSearch[]</code>
	*/
	public WrapperOut(Wrapper[] array) {

		this.records = array ;
	}

// **********************************************************************************************************
// **********************************		ELEMENT MANAGEMENT		*****************************************
// **********************************************************************************************************

	/**
	* Validation is performed by the Eclipse's <code>ADBBean</code> objects (e.g. TownType.setTownType() etc.)
	* If the validation process is succesfull, the resulting <code>ADBBean</code> object is stored as
	* class field to save computation time, since after validation we expect that the method
	* <code>getElement()</code> should be invoked.<br/><br/>
	*
	* In current implementation we DO NOT set the message field to any value, because
	* if everything goes well we don't want to return any message.<br/><br/>
	*
	* <strong>Remark:</strong> to grant LOOSE COUPLING to the WSDL, the <code>ADBBean</code> Java classes
	* should be used only within this method and in the <code>forceMessage()</code> method, not elsewhere.
	*
	* @return <code>true</code> if all the mandatory <code>ADBBean</code> objects have been set correctly
	*/
	public abstract boolean validate() throws WrapperFault ;

	/**
	* Load all the STATIC default values (defined in this class) into the corresponding class attributes.
	* This method should also populate the <code>records</code> array of <code>Wrapper</code> objects
	* as expected by the default empty response.<br/><br/>
	*
	* Example: if the empty SOAP response should contain just one single <code>Wrapper</code> object, this
	* method should create an array with just one record.<br/><br/>
	*
	* <strong>Hint</strong>: subclasses should call this method inside the constructor
	*
	* @see Wrapper()
	* @see forceMessage(String message)
	*/
	public void setDefaults() {

		this.status 		= STATUS_WRONG ;
		this.message 		= MESSAGE_WRONG ;
		this.details		= NO_DETAILS ;
		this.records 		= new Wrapper[0] ;
		// Uncomment next line only if we have minoccurs="1" for this element
		//this.records[0]	= new WrapperSearch() ;
	}

	/**
	* Set an integer value in the wrapped ADBBean element, without checking for validation and ignoring the
	* value of the <code>status</code> class attribute. This method is used when validation fails and it's
	* necessary to set the response status in the status field of the wrapped ADBBean element.
	*
	* @param status the response status value
	*/
	public abstract void forceStatus(int value) ;

	/**
	* Set a string message in the inner <code>ADBBean</code> element, without checking for validation and
	* overwriting the previous value of the <code>message</code> class attribute (if any).
	* This method is used when validation fails and it's necessary to communicate the error in a message field
	* of the wrapped <code>ADBBean</code> element (usually the <em>message</em> field).<br/><br/>
	*
	* <strong>Remark:</strong> this method MUST be called only after validation, because we want to
	* put all the building logic of the <code>ADBBean</code> structure in the <code>validate()</code> method.
	*
	* @param message the error message that must be set in the proper message field of the response
	*/
	public abstract void forceMessage(String message) ;

	/**
	* Cleans the inner wrapper (or the inner array of wrappers), by resetting all fields to their minimal
	* value. This method should be called when the response is wrong or failed, and we want to avoid to
	* return some incomplete (but correct) data to the client.<br/><br/>
	*
	* This method MUST BE called before validation, since it affects the wrapper's class fields and not
	* the <code>ADBBean</code> elements.
	*/
	public void reset() {

		for (Wrapper wrapper: this.records) {
			if (Settings.VERBOSE) System.out.println(leadingName + "Reset wrapper " +  wrapper) ;
		}
	}

// **********************************************************************************************************
// **********************************			ACCESSORS			*****************************************
// **********************************************************************************************************

	/**
	* Return the value of the status flag wrapped by this class.<br/>
	* Default implementation uses <code>1</code> for a good valid SOAP response, <code>0</code> elsewhere.
	* Subclasses can change this definitions by overriding the static <code>STATUS_XXX</code> fields.
	*
	* @return the integer status flag (the wrapper's class field)
	*/
	public int getStatus()								{		return this.status  ;						}

	/**
	* Return the value of the text message wrapped by this class.<br/>
	* The message specifies the type of the SOAP response described by the status flag.
	* Subclasses can change this definitions by overriding the static <code>MESSAGE_XXX</code> fields.
	*
	* @return the text status message (the wrapper's class field)
	*/
	public StringBuffer getMessage()					{		return this.message  ;						}

	/**
	* Return the detailed text message wrapped by this class.<br/>
	* This field adds details to the <code>message</code> text.
	*
	* @return the detailed text status message (the wrapper's class field)
	*/
	public StringBuffer getDetails()					{		return this.details  ;						}

 	/**
	* Return the array of <code>Wrapper</code> records wrapped by this class.<br/>
	* It the web service returns just a single <code>Wrapper</code> object, this should be stored
	* into the <code>records[0]</code> element.
	*
	* @return the array of <code>Wrapper</code> records (the wrapper's class field)
	*/
	public Wrapper[] getRecords()						{		return this.records  ;						}

// **********************************************************************************************************
// **********************************			MUTATORS		*********************************************
// **********************************************************************************************************

	/**
	* Set the value of the status flag wrapped by this class.<br/>
	* Default implementation uses <code>1</code> for a good valid SOAP response, <code>0</code> elsewhere.
	* Subclasses can change this definitions by overriding the static <code>STATUS_XXX</code> fields.
	*
	* @param value the integer status flag (will be stored as class field)
	*/
	public void setStatus(int value)					{		this.status = value ;						}

	/**
	* Set the value of the text message wrapped by this class.<br/>
	* The message details the quality of the SOAP response described by the status flag.
	* Subclasses can change this definitions by overriding the static <code>MESSAGE_XXX</code> fields.
	*
	* @param value the text status message (will be stored as class field)
	*/
	public void setMessage(StringBuffer value)			{		this.message = value ;						}

	/**
	* Set the detailed text message wrapped by this class.<br/>
	* This field adds details to the <code>message</code> text.
	*
	* @param the detailed text status message (the wrapper's class field)
	*/
	public void setDetails(StringBuffer value)			{		this.details = value ;						}

	/**
	* Set the array of <code>Wrapper</code> records wrapped by this class.<br/>
	* It the web service returns just a single <code>Wrapper</code> object, this should be stored
	* into the <code>records[0]</code> element.
	*
	* @param value the array of <code>Wrapper</code> records ((will be stored as class field)
	*/
	public void setRecords(Wrapper[] value)				{		this.records = value ;						}

} // End of class